
package taxman;

import  java.awt.*;

/**
* This is the lower left button panel for the applet.
*  It controls the new game and options buttons and
*  supplies a checkbox for the user to choose to go first.
*
*  @author Jacob Crossman
*  @version 1.0
*/
public class AdvantagePanel extends Panel implements GameUpdate
{
   private static final Font LABEL_FONT  = new Font("Helvetica", Font.BOLD, 12);
   private static final int  MAX_CURRENT = 40;
   private static final int  MAX_FUTURE  = 40;
   private static final int  BAR_SPACING = 2;
   private static final int  PROGRESS_FACTOR = 2;

   //These labels appear above the progress bars which indicate
   // Current Advantage and Future Advantage respectively
   private Label    txt_cur    = new Label("Current Taxman Advantage");
   private Label    txt_future = new Label("Future Taxman Advantage");

   //These progress bars show graphically how far ahead/behind a person
   // is and will be
   private ProgressBar prog_cur = new ProgressBar(0, MAX_CURRENT, true, true);
   private ProgressBar prog_future = new ProgressBar(0, MAX_FUTURE, true, true);


   /**
   * Initializes UI components and sets the layout.
   */
   public AdvantagePanel()
   {

      //Set up font for regular labels
      txt_cur.setFont(LABEL_FONT);
      txt_cur.setAlignment(Label.CENTER);
      txt_future.setFont(LABEL_FONT);
      txt_future.setAlignment(Label.CENTER);

      //Set up both progress bars with given insets and font
      prog_cur.setInsets(new Insets(BAR_SPACING,BAR_SPACING,BAR_SPACING,BAR_SPACING));
      prog_cur.setFont(LABEL_FONT);
      prog_future.setInsets(new Insets(BAR_SPACING,BAR_SPACING,BAR_SPACING,BAR_SPACING));
      prog_future.setFont(LABEL_FONT);

      // Make it look good
      createLayout();
   }

   /**
   * Does the layout fancy work for the applet.
   *  It uses a gridLayout to create its particular look.
   */
   private void createLayout()
   {
      GridBagLayout      gbl = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();

      setLayout(gbl);

      //Constraints for top text
      //Top, left, bottom, right
      gbc.insets    = new Insets(20, 10, 2, 10);

      gbc.anchor    = GridBagConstraints.CENTER;
      gbc.fill      = GridBagConstraints.BOTH;
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      gbc.weightx   = 1.0;
      gbc.weighty   = 0.5;
      gbl.setConstraints(txt_cur, gbc);
      add(txt_cur);

      //Constraints for current advantage bar
      //Top, left, bottom, right
      gbc.insets    = new Insets(2, 10, 2, 10);

      gbc.anchor    = GridBagConstraints.CENTER;
      gbc.fill      = GridBagConstraints.BOTH;
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbl.setConstraints(prog_cur, gbc);
      add(prog_cur);


      //Constraints for future advantage bar
      gbc.anchor    = GridBagConstraints.CENTER;
      gbc.fill      = GridBagConstraints.BOTH;
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      gbc.weightx   = 1.0;
      gbc.weighty   = 0.5;
      gbl.setConstraints(txt_future, gbc);
      add(txt_future);

      //Constraints for future advantage bar
      //Top, left, bottom, right
      gbc.insets    = new Insets(2, 10, 5, 10);

      gbc.anchor    = GridBagConstraints.CENTER;
      gbc.fill      = GridBagConstraints.BOTH;
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbl.setConstraints(prog_future, gbc);
      add(prog_future);


      validate();
   }

   /**
   * Over-ride base function to make sure colors are
   *  propagated to the sub-components
   * @param clr The new foreground color
   */
   public void setForeground(Color clr)
   {
      super.setForeground(clr);

      txt_cur.setForeground(clr);
      txt_future.setForeground(clr);
   }

   /**
   * Over-ride base function to make sure colors are
   *  propagated to the sub-components
   * @param clr The new background color
   */
   public void setBackground(Color clr)
   {
      super.setBackground(clr);

      txt_cur.setBackground(clr);
      txt_future.setBackground(clr);
   }

   /**
   * The standard update method from GameUpdate
   * This one handles NEW_GAME, CPU_CHOSE, HUMAN_CHOSE.
   * @param b The current game board
   * @param m The current minimax object
   * @param flags Indicator of current state
   */
   public void gameUpdate(Board b, Minimax m, int flags)
   {
      if(flags == Game.CPU_CHOSE || flags == Game.HUMAN_CHOSE)
         updateProgress(b,m);
      else if(flags == Game.NEW_GAME)
         resetProgressBars(b);

      validate();
   }

   /**
   * Updates the progress bars when a person or the cpu chooses a number
   * @param b The game board which can supply current advantage info
   * @param m The minimax algorithm to use to collect statistics
   */
   private void updateProgress(Board b, Minimax m)
   {
      //System.out.println("Current Advantage: " + (b.getCPUScore() - b.getHumanScore()));
      //System.out.println("Future Advantage: " + m.getPercievedAdvantage());

      //Put the correct values in the progress bars
      prog_cur.setValue(b.getCPUScore() - b.getHumanScore());
      prog_future.setValue(m.getPercievedAdvantage());

      //make it show up
      repaint();
   }

   /**
   * Sets the status bars to 0.
   * @param b the board which is used to determine the length
   *            of the progress bars
   */
   private void resetProgressBars(Board b)
   {
      //Want the max move which is the last move
      // in the list
      int[] moves = b.getAvailableMoves();

      //Base progress bar on max value of the board
      prog_cur.setMaxValue(Math.round(PROGRESS_FACTOR*moves[moves.length - 1]));
      prog_future.setMaxValue(Math.round(PROGRESS_FACTOR*moves[moves.length - 1]));

      //Default value is 0
      prog_cur.setValue(0);
      prog_future.setValue(0);

      //Repaint the screen
      repaint();
   }

   /**
   * An over-ride of the update function to explicitely
   *  repaint the progress bars (which don't seem to print otherwise)
   * @param g The graphics context on which to draw
   */
   public void update(Graphics g)
   {
     //Repaint progress bars
     prog_cur.repaint();
     prog_future.repaint();
   }
}
