package taxman;

import java.util.*;

/**
* Class Board
*
* The Board represents game state.
* It holds the
*       - list of available numbers
*       - stack of moves made
*       - current turn
*
* It can
*       - Make a move based on a number choice
*       - Make a move based on an index into its list of available numbers
*       - Add a number into its list of available numbers
*       - Undo the last move made
*       - Give an evaluation of current game state (who's winning)
*       - Print out the current game state including
*               + board status
*               + scores
*               + previous moves
*
*/
public class Board extends Object
{
   /** Identifies human turn */
   public static final int HUMAN = 0;
   /** Identifies cpu turn */
   public static final int CPU   = 1;

   private static final int ADD      = 0;
   private static final int SUBTRACT = 1;

   private Vector    board_vec      = new Vector();      //sorted by factors
   private Stack     prev_moves     = new Stack();
   private int       cur_turn       = HUMAN;

   private TaxNumber[] board        = null;              //the board used for speed
   private boolean[]   flags        = null;              //sets moves as usable
   private boolean     boardFilled  = false;             //Board cannot hold more numbers
   private int         avail_moves  = 0;                 //available moves

   private int       human_score    = 0;
   private int       cpu_score      = 0;

   /** Default human starts */
   public Board() {}

   /**
   * Constructor only sets the initial player
   * @param Player Id for the player who will start
   */
   public Board(int Player) { cur_turn = Player; }

   /**
   * This function adds a number to the vector of available moves.
   *  The ordering is by decreasing factor numbers. (overloaded for ints)
   *  Therefore, 13, with 2 factors, comes after 4 with 3 factors
   * @param n The number to add to the available move list
   */
   public final synchronized void addNumber(int n)
   {
      addNumber(new TaxNumber(n));
   }

   /**
   * This function adds a number to the vector of available moves.
   *  The ordering is by decreasing factor numbers.
   *  Therefore, 13, with 2 factors, comes after 4 with 3 factors
   * @param n The number to add to the available move list
   */
   public final synchronized void addNumber(TaxNumber n)
   {
      //The board was already filled and locked.
      // Don't do any more number adding
      if(boardFilled)
         return;

      int i;

      //Sort small to large
      for(i = 0; i < board_vec.size() &&
          n.getValue() > ((TaxNumber)board_vec.elementAt(i)).getValue(); i++);

      //Insert in order
      board_vec.insertElementAt(n, i);
   }

   /**
   * Ends the board filling phase, now a move can be made.
   * If a move is made before calling this function, an exception
   *  will likely occure.  After this function is called, no more
   *  numbers can be added to the game board.  You can make and
   *  and undo moves, that's all.
   */
   public final synchronized void endBoardFill()
   {
      //Set board filled up flag
      boardFilled = true;

      //Create an array for the board and for the
      // USED flags.  The used flags indicate whether
      // a number has been used yet.
      avail_moves = board_vec.size();
      board       = new TaxNumber[avail_moves];
      flags       = new boolean[avail_moves];

      //Fill board and flags for first time
      for(int i = 0; i < avail_moves; i++)
      {
         board[i] = (TaxNumber)board_vec.elementAt(i);
         flags[i] = true;
      }
   }

   /**
   * Removes the given number from the list of available moves.
   * @param n The number to remove from the list of available moves
   * @return If n exists as a valid move, it is removed from the vector
   *   of available moves and returned.  Otherwise, null is returned.
   */
   public final synchronized TaxNumber removeNumber(TaxNumber n)
   {
      for(int i = 0; i < board.length &&
                    (board[i].getValue() <= n.getValue()); i++)
      {
         //See if number exists then remove it if it does.
         // Otherwise it is error time
         if(flags[i] && n.getValue() == board[i].getValue())
         {
            avail_moves--;
            flags[i] = false;        //Cannot use number later
            return board[i];
         }
      }

      System.out.println("Returning null");

      //Not in the list of available moves. Bad choice (by human)
      return null;
   }

   /**
   * Makes the given move (chooses the number and gives the
   *  tax numbers to the opponent). The move and its available
   *  factors are removed from the list of available numbers.
   * @param n The number to choose on making the move
   * @return true if the number is a valid move.  false otherwise.
   */
   public final synchronized boolean makeMove(int n) { return makeMove(new TaxNumber(n)); }

   /**
   * Makes the given move (chooses the number and gives the
   *  tax numbers to the opponent). The move and its available
   *  factors are removed from the list of available numbers.
   * @param n The number to choose on making the move
   * @return true if the number is a valid move.  false otherwise.
   */
   public final synchronized boolean makeMove(TaxNumber n)
   {
      //Try this move
      TaxNumber   choice  = removeNumber(n);

      //Invalid choice (not in board) try another
      if(choice == null)
         return false;

      Vector      taxes   = removeTaxes(choice);

      //Create the move then push it on the stack for undo capabilities
      Move theMove = new Move(cur_turn, choice, taxes);
      prev_moves.push(theMove);

      //Update the score and move to next turn
      updateScore(theMove, ADD);
      changeTurns();

      return true;
   }

   /**
   * Removes all the 'taxes'  or factors associated with a given
   *  number.  A list of the factors, in numeric order, is returned
   * @param choice The number that was chosen and which should be
   *          searched for factors
   */
   private final synchronized Vector removeTaxes(TaxNumber choice)
   {
      Vector      taxes   = new Vector();

      //Remove all of the factors and place in vector ordered
      // Counts on list being numerically ordered
      for(int i = 0; i < board.length &&
                     (choice.getValue() >= board[i].getValue()); i++)
      {
         //isAFactor will return true if the board element is a factor
         // of the player choice.  Flags indicates whether the number is
         // available
         if(flags[i] && choice.isAFactor(board[i]))
         {
            taxes.addElement(board[i]);   //vector for move object
            flags[i] = false;             //set as not usable
            avail_moves--;                //factors cannot be moves either
         }
      }

      //Give back the taxes for the move generator
      return taxes;
   }

   /**
   * Gets the next available move after the given index, n.
   * @param n The index after which to find the available move.
   *           i.e. to find the first move n = -1.
   * @return The index of the next move after n.  If this index
   *          is used in a call to makeIndexedMove() the system
   *          will garrantee a valid move (assuming no other changes).
   */
   public final synchronized int getNextIndexedMove(int n)
   {
      //Search for th next move (from long end)
      for(int i = (board.length - n - 2); i >= 0; i--)
         if(flags[i]) return (board.length - i - 1);

      //No more moves, set to invalid index which
      // could cause an exception
      return -1;
   }

   /**
   * Makes ith possible move at this level.  The total of moves available
   * at this level are given by getNumberOfAvaialbleMoves()
   * @param  index  Index of move that should be tried
   * @return true if the indexed move is value (should be).  false otherwise.
   */
   public final synchronized boolean makeIndexedMove(int index)
   {
      //We index backward from end of the array (for search efficiency)
      int real_index = board.length - index - 1;

      if(flags[real_index])
      {
         //Make move at that index unavailable
         avail_moves--;
         flags[real_index] = false;

         Vector taxes   = removeTaxes(board[real_index]);

         //Create the move then push it on the stack for undo capabilities
         Move theMove = new Move(cur_turn, board[real_index], taxes);
         prev_moves.push(theMove);

         //Update the score and move to next turn
         updateScore(theMove, ADD);
         changeTurns();

         return true;
      }
      else
         return false;
   }


   /**
   * Undoes the last move, putting the board in the same state
   * it was in before last move.
   * @return true if the undo was successful, false otherwise.
   */
   public final synchronized boolean undoLastMove()
   {
      //Can't undo nothing
      if(prev_moves.empty())
         return false;

      //Retrieve the last move
      Move old_move = (Move)prev_moves.pop();

      int i = 0;
      TaxNumber cur_num = null;

      //Return taxes collected
      Enumeration taxes = old_move.getTaxList().elements();
      while(taxes.hasMoreElements())
      {
         //Grab a number (factor)
         cur_num = (TaxNumber)taxes.nextElement();

         //Find that number in the list and set it to
         // usable, we know the factors are sorted ascending,...
         for(; i < board.length; i++)
         {
            if(!flags[i] && board[i].getValue() == cur_num.getValue())
            {
               avail_moves++;            //more moves can be made again
               flags[i] = true;          //number usable now
               break;
            }
         }
      }

      //Get the actual move last (garranteed to be largest
      // number and farthest along in the array)
      cur_num = old_move.getMove();

      //Find that number in the list and set it to usable
      for(; i < board.length; i++)
      {
         if(!flags[i] && board[i].getValue() == cur_num.getValue())
         {
            avail_moves++;                //more moves can be made again
            flags[i] = true;              //usable again
            break;                        //found last number, stop looking
         }
      }

      //Change the score back and switch turns
      updateScore(old_move, SUBTRACT);
      changeTurns();

      return true;
   }

   /**
   * The evaluation function for the board.  It is very simple.
   *  It's evaluation turns out to just be the difference in scores
   *  at this time.  The bigger the difference, the better the state.
   * @return An integer representing the quality of this move
   */
   public final synchronized int evaluate()
   {
      return (cur_turn == CPU)? cpu_score - human_score: human_score - cpu_score;
   }


   /**
   * Updates the current score of the game.
   * @param m The move that was just made
   * @param type The type of move (an undo or foward move)
   */
   private final synchronized void updateScore(Move m, int type)
   {
      if(type == ADD)
      {
         //Going foward, add points to each score
         cpu_score   += (m.getPlayer() == CPU)? m.scoreMade(): m.taxesCollected();
         human_score += (m.getPlayer() == CPU)? m.taxesCollected(): m.scoreMade();
      }
      else if(type == SUBTRACT)
      {
         //Going backward (undo) subtract points from each score
         cpu_score   -= (m.getPlayer() == CPU)? m.scoreMade(): m.taxesCollected();
         human_score -= (m.getPlayer() == CPU)? m.taxesCollected(): m.scoreMade();
      }
   }

   /**
   * Returns the move stored at the index i (the way this board stores moves
   *  could be different from other boards, but the effect is the same)
   * @return The move (as an object) at the given index
   */
   public final Object getMoveAtIndex(int i) { return board[board.length - i - 1]; }

   /**
   * Returns the number of available moves (just the number of choices the
   *  the player has at this time)
   * @return The number of moves available at the current state
   */
   public final int getNumberOfAvailableMoves() { return avail_moves; }

   /**
   * Returns the last move made.
   * @return the last move made (from the stack)
   */
   public final Move getLastMove() { return ((Move)prev_moves.peek()); }

   /**
   * Returns the the player who has the current move.
   * @return The value Board.HUMAN if it is the human's turn
   *         The value Board.CPU if it is the cpu's turn
   */
   public final int getCurrentTurn() { return cur_turn; }
   /** Switches turns (internal state */
   private final synchronized void changeTurns() { cur_turn = (cur_turn == CPU)? HUMAN: CPU; }

   /**
   * Returns the winner ID (if the game ended at this state
   * @return The winner's id
   */
   public final int getWinner() { return (cpu_score > human_score)? CPU: HUMAN; }

   /**
   * Returns the current CPU score
   * @return The current CPU score
   */
   public final int getCPUScore() { return cpu_score; }

   /**
   * Returns the current Human score
   * @return the current Human score
   */
   public final int getHumanScore() { return human_score; }

   /**
   * Retrieves a list of numerically ordered numbers that are available for choosing.
   * @return An array of numbers sorted numerically indicating the numbers that are
   *          available for choosing.
   */
   public final synchronized int[] getAvailableMoves()
   {
      if(avail_moves < 1)
         return null;

      //Create an array which is easier to handle outside this object
      int[] ret_array = new int[avail_moves];

      //For each move that is usable, add it to the
      // return vector
      for(int i = 0, j = 0; i < board.length; i++)
         if(flags[i])
            ret_array[j++] = board[i].getValue();

      return ret_array;
   }

   /**
   * Displays current state of board, score, and other things.
   */
   public final void displayState()
   {
      //Print Scores
      System.out.println("-- SCORE -------------------------------------------");
      System.out.println("Computer: " + cpu_score + ";  You: " + human_score);
      System.out.println("Current Player: " + ((cur_turn == CPU)? "CPU": "YOU"));
      System.out.println("Current Player Advantage: " + evaluate());

      //Print Move List
      Stack tmp = new Stack();
      System.out.println("\n-- PREVIOUS MOVES (last shown first) ---------------");
      while (!prev_moves.empty())
      {
         System.out.println((Move)prev_moves.peek());
         tmp.push(prev_moves.pop());
      }

      //Push everything back on
      while (!tmp.empty())
         prev_moves.push(tmp.pop());

      ///////////////////////////////////////////////////////////////////////////////////

      System.out.println("\n-- AVAILABLE NUMBERS (choose one)-------------------");
      System.out.println("# of available numbers: " + getNumberOfAvailableMoves());

      int[]        move     = getAvailableMoves();

      StringBuffer moves    = new StringBuffer();
                   moves.append("[ ");

      for(int i = 0; move != null && i < move.length; i++)
         moves.append(move[i] + " ");

      //Close out list of available numbers
      moves.append("]");
      System.out.println(moves.toString());
   }

   /** Test function. */
   public static void main(String[] args)
   {
      Board board = new Board();

      for(int i = 1; i < 13; i++)
         board.addNumber(i);

      board.endBoardFill();

      board.displayState();

      int n = board.getNextIndexedMove(-1);
      board.makeIndexedMove(n);
      board.displayState();

      n = board.getNextIndexedMove(n);
      board.makeIndexedMove(n);
      board.displayState();

      n = board.getNextIndexedMove(n);
      board.makeIndexedMove(n);
      board.displayState();
   }

}
