
package taxman;

import  java.awt.*;

/**
* This is the lower left button panel for the applet.
*  It controls the new game and options buttons and
*  supplies a checkbox for the user to choose to go first.
*
*  @author Jacob Crossman
*  @version 1.0
*/
public class ButtonPanel extends Panel implements GameUpdate
{
   private static final Font LABEL_FONT = new Font("Helvetica", Font.BOLD, 12);

   private Button   new_game = null;
   private Button   options  = null;
   private Checkbox go_first = null;
   private Label    chk_lbl  = new Label("Go First");

   private Game     control  = null;

   /**
   * Initializes UI components and sets the game control alias.
   * @param g The game control object that will be called back
   *           on user choices.
   */
   public ButtonPanel(Game g)
   {
      new_game = new Button("New Game");
      options  = new Button("Options");

      //Initial state of checkbox determined by game defaults
      go_first = new Checkbox("");
      go_first.setState(!Game.DEF_CPU_FIRST);

      chk_lbl.setFont(LABEL_FONT);
      chk_lbl.setAlignment(Label.RIGHT);

      //Sets the control so that this panel can send
      // update message back to the game control
      control = g;

      // Make it look good
      createLayout();
   }

   /**
   * Does the layout fancy work for the applet.
   *  It begins as a grid layout but may change to gridbag later.
   */
   private void createLayout()
   {
      GridBagLayout      gbl = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();

      setLayout(gbl);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(20,20,10,2);

      //Puts the checkbox on the top
      gbc.anchor = GridBagConstraints.EAST;
      gbc.fill   = GridBagConstraints.HORIZONTAL;
      gbc.gridx  = 1;
      gbc.gridy  = 1;
      gbc.weightx = 1.0;
      gbc.weighty = 0.5;
      gbl.setConstraints(chk_lbl, gbc);
      add(chk_lbl);

      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(20,2,10,20);

      //Puts the checkbox on the top
      gbc.anchor = GridBagConstraints.EAST;
      gbc.fill   = GridBagConstraints.HORIZONTAL;
      gbc.gridx  = 2;
      gbc.gridy  = 1;
      gbc.weightx = 1.0;
      gbc.weighty = 0.5;
      gbl.setConstraints(go_first, gbc);
      add(go_first);

      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(0,20,5,20);

      //Puts the new game button in the middle
      gbc.anchor = GridBagConstraints.CENTER;
      gbc.fill   = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 1;
      gbc.gridy     = 2;
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      gbc.weightx   = 0.5;
      gbc.weighty   = 0.2;
      gbl.setConstraints(new_game, gbc);
      add(new_game);

      //Puts the option button lowest
      gbc.anchor = GridBagConstraints.CENTER;
      gbc.fill   = GridBagConstraints.HORIZONTAL;
      gbc.gridx  = 1;
      gbc.gridy  = 3;
      gbc.weightx = 0.5;
      gbc.weighty = 0.2;
      gbl.setConstraints(options, gbc);
      add(options);
   }


   /**
   * Over-rides setBackground to set the internal components
   *  backgrounds to match
   * @param clr The color to set the background to
   */
   public void setBackground(Color clr)
   {
      super.setBackground(clr);
      
      go_first.setBackground(clr);
      chk_lbl.setBackground(clr);

      //Return buttons color
      new_game.setBackground(Color.lightGray);
      options.setBackground(Color.lightGray);
   }

   /**
   * Over-rides setForeground to set the internal components
   *  foregrounds to match
   * @param clr The color to set the foreground to
   */
   public void setForeground(Color clr)
   {
      super.setForeground(clr);
      go_first.setForeground(clr);
      chk_lbl.setForeground(clr);

      //Don't change buttons
      new_game.setForeground(Color.black);
      options.setForeground(Color.black);
   }


   /**
   * The standard update method from GameUpdate
   * This one handles OPTIONS_OPEN, OPTIONS_CLOSE.
   * @param b The current game board
   * @param m The current minimax object
   * @param flags Indicator of current state
   */
   public void gameUpdate(Board b, Minimax m, int flags)
   {
      if(flags == Game.OPTIONS_OPEN)
      {
         //When we go into options, the buttons
         // are no longer valid
         new_game.disable();
         options.disable();
      }
      else if(flags == Game.OPTIONS_CLOSE)
      {
         //When they press cancel or ok
         // the buttons are then valid again
         new_game.enable();
         options.enable();
      }

      repaint();
   }

   /**
   * This method traps the events for the two buttons
   *  and reports them back to the game control.
   * @param evt The event that occured
   * @param obj The object that was effected.
   */
   public boolean action(Event evt, Object obj)
   {
      // If the new game button was pressed,
      //  then start a new game (call control function
      if(evt.target == new_game)
         control.newGame(go_first.getState());

      // If the options button was pressed, go to the
      //  announce to the game control which will notify
      //  the panels that need to change
      if(evt.target == options)
         control.changeOptionsScreen(true);

      return true;
   }

}
