package taxman;

import java.awt.*;
import java.util.*;

/**
* This is the central choosing grid. It can handle mouse clicks
*  and report them back to the Game control
*
* @author Jacob Crossman
* @version 1.0
*/
public class ChoiceTable extends GameTable
{
   private boolean   mouseEnabled = false;

   private Image     backImage    = null;
   private Color     numClr       = DEF_NUM_COLOR;
   private Color     backClr      = Color.gray;

   private Game      control      = null;

   /**
   * The constructor takes many parameters and builds up a table which
   *  can respond to user clicks.  The response is sent to the Controlling
   *   game object that is passed in.
   * @param num_color The color of the numbers that are displayed
   * @param backGround The background image for the number (can be null)
   * @param g The game control object which can recieve numberChosen messages
   */
   public ChoiceTable(Color num_color, Image backGround, Game g)
   {
      super();                   //Call base constructor to do some initial stuff

      numClr    = num_color;     //Set some look properties
      backImage = backGround;
      control   = g;
   }

   /**
   * The standard update method from GameUpdate
   * This one handles NEW_GAME, HUMAN_MOVE, CPU_MOVE, CPU_CHOSE, HUMAN_CHOSE.
   * @param b The current game board
   * @param m The current minimax object
   * @param flags Indicator of current state
   */
   public void gameUpdate(Board b, Minimax m, int flags)
   {
      super.gameUpdate(b,m,flags);                   //base class does some stuff

      if(flags == Game.NEW_GAME)                     //re-fill with numbers
      {
            fillNumbers(b);
      }
      else if(flags == Game.HUMAN_MOVE)              //let human choose
      {
         mouseEnabled = true;
      }
      else if(flags == Game.CPU_MOVE)                //disable human choices
      {
         mouseEnabled = false;
      }                                              //make an actual number change
      else if(flags == Game.CPU_CHOSE || flags == Game.HUMAN_CHOSE)
      {
         removeNumbersChosen(b);
      }
   }

   /**
   * Fills the grid with numbers that are available as moves from the board.
   * @param b The current (should be new) game board.
   */
   private void fillNumbers(Board b)
   {
      //Grab the list of numbers available from the board
      int[] avail_nums = b.getAvailableMoves();

      //Loop through and fill 'em
      for(int i = 0; i < avail_nums.length; i++)
      {
         if(backImage != null)
            addNumber(new GUINumber(avail_nums[i], numClr, backImage));
         else
            addNumber(new GUINumber(avail_nums[i], numClr, backClr));
      }
   }

   /**
   * Removes the chosen number and it 'taxes' from the grid
   * @param b The current game board (can grab the last move from it)
   */
   private void removeNumbersChosen(Board b)
   {
      Move last_move = b.getLastMove();

      //Remove the number that was actually chosen
      removeNumber(last_move.scoreMade());

      //Remove all the 'taxes' (or factors)
      Enumeration e = (last_move.getTaxList()).elements();
      while(e.hasMoreElements())
         removeNumber(((TaxNumber)e.nextElement()).getValue());
   }

   /**
   * Again, using the old event model.  We use mouse up events
   *  to capture selection  of numbers in the grid.
   * @param evt the mouse event
   * @param x the x location of the mouse
   * @param y the y location of the mouse
   * @return true to allow event propogation
   */
   public boolean mouseUp(Event evt, int x, int y)
   {
      // Loop through all the numbers to see if one was
      //  selected by the click.  If so, notify control
      for(int i =  0; i < numbers.length; i++)
      {
         //The check for numbers[i] = null is important so that
         // the numbers that were removed do not cause an exception
         if(numbers[i].getSelectionState())
         {
            if(mouseEnabled && control != null)
               control.numberChosen(numbers[i].getNumber());
            else
               numbers[i].setSelectionState(false);      //Don't overselect when multi-threading

            break;
         }
      }

      return true;                                       //Enable event propagation
   }
}
