package taxman;

import java.awt.*;

/**
* This class is the little button-like numbers that the user
*  may select to indicate a number choice in the game.
*/
class GUINumber extends Canvas
{
   /** Default values for a couple of properties */
   private static final Color DEF_BACK       = Color.white;
   private static final Color DEF_FORE       = Color.black;
   private static final Font  DEF_FONT       = new Font("Helvetica",Font.BOLD,16);
   private static final Point DEF_INSETS     = new Point(2,2);
   private static final int   DOWN_SHIFT     = 5;

   private int    num = 0;                      //The numeric value displayed in the center
   private Image  img = null;                   //The image to use as background (null = no image)


   private Color  fore_clr = null;              //Foreground Color
   private Color  back_clr = null;              //Background Color
   private Point  insets   = DEF_INSETS;        //Insets or space around outside of number
   private Font   fnt      = DEF_FONT;          //Font of number

   private boolean isSelected = false;          //True if the object was selected by mouse
   private boolean selEnabled = false;          //True if selection is enabled for this number


   /**
   * Constructor which creates a panel with a number in the middle. This
   *  version creates a number with a flat fill background.
   * @param number The number to place in the middle
   * @param foreColor The color of the number
   * @param backGround The color of the background if there is no image
   *         or the image is transparent
   */
   public GUINumber(int number, Color foreGround, Color backGround)
   {
      num = number;            //set the number
      img = null;              //Background image not here

      fore_clr = foreGround;   //user defined number color
      back_clr = backGround;   //user defined background color
   }

   /**
   * Constructor which creates a panel with a number in the middle. This
   *  version creates a number with an image as a background.
   * @param number The number to place in the middle
   * @param foreColor The color of the number
   * @param backGround The image (should be nearly square) that serves
   *           as the background for the number
   */
   public GUINumber(int number, Color foreGround, Image backGround)
   {
      num = number;                 //set the number
      img = backGround;             //Background image

      fore_clr = foreGround;        //user defined number color
      back_clr = getBackground();   //background color is defaulted here
   }


   /**
   * Draws the number with its background image (if it exists).
   *  If no background image exists, a flat fill is used
   * @param g the graphics device context for this object
   */
   public void paint(Graphics g)
   {
      //Didn't work, try Java 1.0 (bounds())
      drawBackground(g, bounds());
      drawNumber(g, bounds());
   }

   /**
   *  Paints the background of this object.  The background may be an image
   *  or a flat-fill as specified in the constructor.
   *  @param g The graphics context to which to draw
   *  @param bounds The bound of this object
   */
   private void drawBackground(Graphics g, Rectangle bounds)
   {
      //Case 1: There is no image
      if(img == null)
      {
         //use user defined background color
         g.setColor(back_clr);
         //draw a rectangle to fill the background (no background is auto-painted)
         g.fillRect(insets.x, insets.y, bounds.width - 2*insets.x, bounds.height - 2*insets.y);
      }
      else
      {
         int img_height = img.getHeight(this);        //Get the dimensions of the image
         int img_width  = img.getWidth(this);         //The image should be loaded fully

         try
         {
            //Draw the image, stretching it to fit.
            g.drawImage(img, insets.x, insets.y, bounds.width - 2*insets.x, bounds.height - 2*insets.y,
                             0, 0, img_width, img_height,
                             this);
         }
         catch (NoSuchMethodError e)
         {
            //Netscape needs this method instead
            g.drawImage(img, insets.x, insets.y, bounds.width - 2*insets.x, bounds.height - 2*insets.y, this);
         }
      }
   }

   /**
   * Draws the number in the center of the box. This number's font and color
   *  can be changed by other method calls.
   * @param g the device context to which to draw
   * @param bounds The size of this component
   */
   private void drawNumber(Graphics g, Rectangle bounds)
   {
      g.setColor(fore_clr);                       //use user defined color

      int center_x = bounds.width / 2;            //calculate the center
      int center_y = bounds.height / 2;

      FontMetrics fm = getFontMetrics(fnt);       //get font info for sizing

      g.setFont(fnt);                             //set user defined font
      g.drawString(Integer.toString(num),         //draw it centered
                   center_x - (fm.stringWidth(Integer.toString(num)) / 2) - 1,
                   center_y + (fm.getAscent() / 2) + DOWN_SHIFT);
   }

   /**
   * Using the old event model to remain compatable with Java 1.0
   *  This is the mouse UP event which will select a given number.
   *  Selection doesn't visibly change the object.  To de-select
   *  an object use selectionOff()
   * @param e The incoming event
   * @param x The x location of the mouse
   * @param y The y position of the mouse
   * @return false, to garrantee propagation of the event
   */
   public boolean mouseUp(Event e, int x, int y)
   {
      if(selEnabled)
         setSelectionState(true);

      //Garrantee that event is propagated higher in the
      // composition tree
      return false;
   }

   /**
   * Gets the selection state of the number object
   * @return true if the number was selected by the user, false otherwise.
   */
   public boolean getSelectionState() { return isSelected; }

   /**
   * Enables/Disables selection from mouse clicks on the number.
   * @param state true to set the selected state to true, false otherwise
   */
   public void setSelectionState(boolean state)
   {
      isSelected = state;
   }

   /**
   * Turns off the ability of the number to be selected.
   */
   public void disableSelection()
   {
      setSelectionState(false);
      selEnabled = false;
   }

   /**
   * Turns on the ability of the number to be selected
   */
   public void enableSelection()
   {
      selEnabled = true;
   }

   /**
   * Prevents a repaint of the background.
   * @param g the graphics context used for drawing
   */
   public void update(Graphics g)
   {
      paint(g);
   }

   /**
   * Gets the number in this panel
   * @return The number displayed on this panel
   */
   public int getNumber() { return num; }

   /**
   * Sets the number that is displayed on this panel.
   * @param n The number to display on this panel.
   */
   public void setNumber(int n) { num = n; }

   /**
   * Sets the font of the number.  There is a default also
   * @param new_font the new font to use for this number
   */
   public void setFont(Font new_font)
   {
      fnt = new_font;
   }

   /**
   * Sets the flat-fill background of this number
   * @param clr The new flat-fill color to use for this number
   */
   public void setBackColor(Color clr)
   {
      back_clr = clr;
   }

   /**
   * Sets the background image for this number.
   * @param image the background image to use for this number.
   */
   public void setBackgroundImage(Image image)
   {
      img = image;
   }

   /**
   * Gets the background image for this number.
   * @return the current background image for this number.
   */
   public Image getBackgroundImage()
   {
      return img;
   }

   /**
   * Gets the flat-fill background color for this number
   * @return The flat-fill background color
   */
   public Color getBackColor()
   {
      //Create a new color object so the user cannot effect the internals
      // without a call to setBackColor()
      return (back_clr != null)? new Color(back_clr.getRed(), back_clr.getGreen(), back_clr.getBlue()): null;
   }

   /**
   * Sets the foreground (number) color
   * @param clr The color to make the number
   */
   public void setForeColor(Color clr)
   {
      fore_clr = clr;
   }

   /**
   * Gets the color of the number
   * @return the foreground (or number) color
   */
   public Color getForeColor()
   {
      //Create a new color object so the user cannot effect the internals
      // without a call to setForeColor()
      return new Color(fore_clr.getRed(), fore_clr.getGreen(), fore_clr.getBlue());
   }

   /**
   * Sets the insets (space around the number box) for this number.  The x and
   *  y directions are
   *  @param x the space to the left and right of the box in pixels
   *  @param y the space to the top and bottom of the box in pixels
   */
   public void setInsets(int x, int y)
   {
      insets = new Point(x,y);
   }

   /**
   * Gets the current insets (space around the box) for this number
   * @return A point with x representing the space to the left and right of the box
   *  and y representing the space above and below the box.
   */
   public Point getInsets()
   {
      return new Point(insets.x, insets.y);
   }
}
