package taxman;

import  java.util.*;

/**
* This is the control class for the game.
*  It has no UI logic and it's only job is to
*  send events to the various panels interested
*  in the game progress.
*
*  @author Jacob Crossman
*  @version 1.0
*/
public class Game extends Object
{
   public  static final int      HUMAN_MOVE     = 0;
   public  static final int      CPU_MOVE       = 1;
   public  static final int      HUMAN_CHOSE    = 2;
   public  static final int      CPU_CHOSE      = 3;
   public  static final int      NEW_GAME       = 10;
   public  static final int      GAME_OVER      = 11;
   public  static final int      OPTIONS_OPEN   = 20;
   public  static final int      OPTIONS_CLOSE  = 21;
   public  static final int      SOUND_ON       = 22;
   public  static final int      SOUND_OFF      = 23;

   public  static final int      DEF_NUMS_USED   = 21;
   public  static final boolean  DEF_RANDOM      = false;
   public  static final boolean  DEF_CPU_FIRST   = false;
   public  static final int      DEF_LOOK_AHEAD  = 5;
   public  static final boolean  DEF_USE_CUTOFF  = true;
   public  static final boolean  DEF_SOUND_STATE = true;

   private Board   board;                                 //game board alias
   private Minimax minimax;                               //minimax algorithm alias

   private int     nums_used  = DEF_NUMS_USED;            //quantity of numbers used in game
   private boolean use_random = DEF_RANDOM;               //whether or not random number are used
   private int     look_ahead = DEF_LOOK_AHEAD;           //depth of computer search
   private boolean use_cutoff = DEF_USE_CUTOFF;           //alpha beta cutoff flag
   private boolean hum_first  = !DEF_CPU_FIRST;           //whether or not the human goes first

   private Vector  panels            = new Vector();      //list of panels interested in the game

   /** Constructor does nothing. */
   public Game() {}

      /**
      * Control function to start a new game.
      * @param first true if the human should go first. false if the CPU goes first
      */

   public synchronized void newGame(boolean first)
   {
      // In case a CPU search is occuring in another thread,...
      if(minimax != null)
         minimax.quit();

      //Set human first flag
      hum_first = first;

      //Create new board and minimax objects
      board   = new Board( (hum_first)? Board.HUMAN: Board.CPU );
      minimax = new Minimax( board, look_ahead, use_cutoff, this );

      //Fill the board with numbers
      if(use_random)
         loadRandom();
      else
         loadInOrder();

      //Lock board so it can begin play
      board.endBoardFill();

      //board.displayState();

      //Update all displays
      updatePanels(NEW_GAME);

      if(first)
         updatePanels(HUMAN_MOVE);                       //let human choose
      else
      {
         updatePanels(CPU_MOVE);
         (new Thread( minimax )).start();                //begin CPU search
      }
   }


   /**
   * A panel can report that a number is chosen (by a human)
   * @param n the number that was chosen
   */
   public synchronized void numberChosen(int n)
   {
      // Set CPU_MOVE
      updatePanels(CPU_MOVE);

      // Make actual move
      board.makeMove(n);

      //board.displayState();

      // Send HUMAN_CHOSE message
      updatePanels(HUMAN_CHOSE);

      if(board.getNumberOfAvailableMoves() <= 0)            // If game is over
         updatePanels(GAME_OVER);                           //    Send GAME_OVER
      else
         (new Thread( minimax )).start();                   //    Run minimax
   }

   /**
   * Called by the option setting panel, this function changes game options.
   * Changing options will require a new game so the panel listing options
   *  should be de-activated during game play.
   * @param a_b flag indicating whether alpha/beta pruning should be used (default = true)
   * @param max_levels number of look-ahead levels for the CPU to search
   * @param quantity the quantity of numbers to put in the game
   * @param random whether or not to use random numbers (otherwise they are sequential
   * @param sound True if the sound is turned on
   */
   public synchronized void optionsChanged(boolean a_b, int max_levels, int quantity, boolean random, boolean sound)
   {
      use_cutoff = a_b;
      look_ahead = max_levels;
      nums_used  = quantity;
      use_random = random;

      if(sound)
         updatePanels(SOUND_ON);
      else
         updatePanels(SOUND_OFF);
   }

   /**
   * A method to update panels when the options button is pressed.
   * @param optionsOpening true if the options dialog is opening, false if it is closing
   */
   public synchronized void changeOptionsScreen(boolean optionsOpening)
   {
      updatePanels((optionsOpening)? OPTIONS_OPEN: OPTIONS_CLOSE);
   }

   /**
   * Makes a move for the CPU and alternates control to the human
   * @param n The number the CPU chose (in a different thread)
   */
   public synchronized void makeCPUMove(int n)
   {
      // Make actual move
      board.makeMove(n);

      // Send CPU_CHOSE message
      updatePanels(CPU_CHOSE);

      //board.displayState();

      if(board.getNumberOfAvailableMoves() <= 0)    // If game is over
         updatePanels(GAME_OVER);                   //    Send GAME_OVER
      else
         updatePanels(HUMAN_MOVE);                  //    Send HUMAN_MOVE
   }

   /**
   * Adds a panel to be updated when the game state changes
   * @param panel The panel (using the interface - GameUpdate) that wants
   *               to know when the game state changes.
   */
   public void addPanel(GameUpdate panel)
   {
      panels.addElement(panel);
   }

   /**
   * Loops through every panel interested in the game and
   *  sends them an update message.
   */
   private void updatePanels(int status)
   {
      for(Enumeration e = panels.elements(); e.hasMoreElements(); )
         ((GameUpdate)e.nextElement()).gameUpdate(board, minimax, status);
   }

   /**
   * Loads the board with random numbers.
   * Numbers are between 1 and the max numbers in board.
   * Duplicates are allowed.
   */
   private void loadRandom()
   {
      // Random number object created
      Random r = new Random(System.currentTimeMillis());

      // Generate random numbers from 1 - 2n
      //  The quantity is given by nums_used
      for(int i = 0; i < nums_used; i++)
      {
         double tmp = (r.nextDouble() * nums_used * 2) + 1;
         board.addNumber((int)Math.round(tmp));
      }
   }

   /**
   * Loads the board with sequential numbers up to the number
   * of numbers in the board.
   */
   private void loadInOrder()
   {
      //Standard sequential (1,2,3,4,...) load
      for(int i = 1; i <= nums_used; i++)
         board.addNumber(i);
   }
}

