
package taxman;

import java.applet.*;

/**	
* This object controls the game sounds.
*
* @Author:	Jacob Crossman
* @Version:	1
*/
public class GameSound extends Object implements GameUpdate
{
   private static final int   INFINITY    = 99999999;
   private static final int   LONG_DELAY  = 1000;
   private static final int   SHORT_DELAY = 200;
   private static final float GOOD_FACTOR = (float)0.0;
   private static final float BAD_FACTOR  = (float)0.6;
   private static final int   DONE        = 0;
   private static final int   MIDDLE      = 1;
   private static final int   BEGIN       = 2;
   private static final int   REUSE_TURNS = 4;            //A sound is reusable after this many turns
   private static final int   REUSABLE    = 0;            //State when sound is reusable

   private boolean soundEnabled   = Game.DEF_SOUND_STATE;
   private int     old_score      = INFINITY;             //old score difference to calc change
   private int     old_adv        = INFINITY;             //old cpu advantage (could be negative)
   private int     big_change     = INFINITY;             //definition of a big change
   private int     playing_sound  = DONE;
   private int[]   reusable       = null;

   //Index names to make sound playing more readable
   private static final int CHOICE    = 0;
   private static final int HUMAN_WON = 1;
   private static final int CPU_WON   = 2;
   private static final int TIE       = 3;
   private static final int NEG_POS   = 4;
   private static final int POS_NEG   = 5;
   private static final int BIG_NEG   = 6;
   private static final int BIG_POS   = 7;
   private static final int NEW_GAME  = 8;


   //Array of sounds to load
   private MediaRecord[]   media_list = {
      //new MediaRecord("taxman/coins2.au", MediaRecord.GRAPHIC),
      //new MediaRecord("taxman/bag_flying.gif", MediaRecord.GRAPHIC),
      //new MediaRecord("taxman/winner.gif", MediaRecord.GRAPHIC),
      new MediaRecord("taxman/coin_good.au", MediaRecord.AUDIO),
      new MediaRecord("taxman/recount_good.au", MediaRecord.AUDIO),
      new MediaRecord("taxman/never_beat_good.au", MediaRecord.AUDIO),
      new MediaRecord("taxman/fathers_day_good.au", MediaRecord.AUDIO),
      new MediaRecord("taxman/have_now_good.au", MediaRecord.AUDIO),
      new MediaRecord("taxman/smart_good.au", MediaRecord.AUDIO),
      new MediaRecord("taxman/backside_good.au", MediaRecord.AUDIO),
      new MediaRecord("taxman/stealing_good.au", MediaRecord.AUDIO),
      new MediaRecord("taxman/challenge_good.au", MediaRecord.AUDIO)
   };

   /** Sets initial values for sound frequency array (limits frequency of
   *   phrase use
   */
   public GameSound ()
   {
      reusable = new int[media_list.length];
   }

   /**
   * This is the common update method for all GameUpdate objects.
   *  It's job is to keep the sounds synchronized with game state.
   * @param b The current game board
   * @param m The current minimax object
   * @param flag a flag indicating the game state
   */
   public void gameUpdate(Board b, Minimax m, int flags)
   {
       //Handles enabling and disabling sounds
       if(flags == Game.SOUND_ON)
          soundEnabled = true;
       else if(flags == Game.SOUND_OFF)
          soundEnabled = false;
       else if(flags == Game.NEW_GAME)
            resetStats();

       //Handles sound events (like choices, etc,..)
       if(soundEnabled)
       {
         if(flags == Game.CPU_CHOSE || flags == Game.HUMAN_CHOSE)
            numberChosen();
         else if(flags == Game.NEW_GAME)
            newGame();
         else if(flags == Game.GAME_OVER)
            gameOver(b.getHumanScore(), b.getCPUScore());

         if(flags == Game.HUMAN_CHOSE && b.getNumberOfAvailableMoves() > 0)
            humanChose(b);
         else if(flags == Game.CPU_CHOSE && b.getNumberOfAvailableMoves() > 0)
            cpuChose(m);
       }
   }

   /**
   * Plays the bag of coins sound and does some
   *  internal book-keeping to keep sound delays good
   */
   private void numberChosen()
   {
      playClip(CHOICE);

      //Notes when most game sounds are played
      // so that delay times can be correct
      if(playing_sound == BEGIN)
         playing_sound = MIDDLE;
      else if(playing_sound == MIDDLE)
         playing_sound = DONE;

      //Basically, reusable acts as a timer indicating
      // when a sound can be used again
      for(int i = 0; i < reusable.length; i++)
          if(reusable[i] > 0) reusable[i]--;
   }

   /**
   * Play Taxman's challenge to the human at beginning
   *  of game
   */
   private void newGame()
   {
      delay();
      playClip(NEW_GAME);
   }

   /**
   * Reset the statistics so next game will have proper sounds
   */
   private void resetStats()
   {
      // Reset the numbers for next game
      old_score      = INFINITY;
      old_adv        = INFINITY;
      big_change     = INFINITY;

      //Resent frequency counter array
      for(int i = 0; i < reusable.length; i++)
         reusable[i] = REUSABLE;
   }

   /**
   * Handles sounds for cpu choice (mainly look-ahead stuff).
   * @param m The minimax object from which to glean look_ahead info
   */
   private void cpuChose(Minimax m)
   {
      if(old_adv != INFINITY)
      {
         //If the cpu was ahead (in future) and Human went ahead
         // then play this sound
         if((old_adv >= 0 && m.getPercievedAdvantage() < 0) &&
             reusable[POS_NEG] == REUSABLE)
         {
            delay();
            playClip(POS_NEG);

            //Keep from playing sounds too often
         }
         //If human was ahead in future, and cpu went ahead
         // then play this sound
         else if((old_adv <= 0 && m.getPercievedAdvantage() > 0) &&
                 reusable[NEG_POS] == REUSABLE)
         {
            delay();
            playClip(NEG_POS);

            //Keep from playing sounds too often
            reusable[NEG_POS] = REUSE_TURNS;
         }
      }

      //Update current advantage for next choice comparison
      old_adv  = m.getPercievedAdvantage();
   }

   /**
   * Handles sounds for human choice.
   * @param b board object which supplies parameters for calculating changes
   */
   private void humanChose(Board b)
   {
      if(old_score == INFINITY)
      {
         //Base a BIG move on how large the largest number is
         int [] nums = b.getAvailableMoves();

         old_score = b.getHumanScore() - b.getCPUScore();
         big_change = nums[nums.length - 1];
      }
      else
      {
         //Get the new difference in scores
         int new_score = b.getHumanScore() - b.getCPUScore();

         //If the human did better by a factor "GOOD_FACTOR" then
         // play the sound for Big positive move
         if((((float)new_score - old_score) > (GOOD_FACTOR*big_change)) &&
             reusable[BIG_POS] == REUSABLE)
         {
            delay();
            playClip(BIG_POS);

            //Keep from playing sounds too often
            reusable[BIG_POS] = REUSE_TURNS;
         }
         //If the human did worse by a factor "BAD_FACTOR" then
         // play the sound for Big negative move
         else if((((float)old_score - new_score) > (BAD_FACTOR*big_change)) &&
                 reusable[BIG_NEG] == REUSABLE)
         {
            delay();
            playClip(BIG_NEG);

            //Keep from playing sounds too often
            reusable[BIG_NEG] = REUSE_TURNS;
         }

         //Update the score for next time
         old_score = new_score;
      }
   }

   /**
   * Plays sounds for game over, if human wins, cpu wins
   *  or if there is a tie.
   * @param human Human score
   * @param cpu Cpu score
   */
   private void gameOver(int human, int cpu)
   {
      //Tie
      if(human == cpu)
      {
         delay();
         playClip(TIE);
      } //CPU won
      else if(human < cpu)
      {
         delay();
         playClip(CPU_WON);
      } //Human won
      else
      {
         delay();
         playClip(HUMAN_WON);
      }
   }

   /**
   * Delays the thread for a specified period allowing
   *  other sounds to finish before this one starts.
   */
   private void delay()
   {
      try
      {
         //If we are in the middle of another
         // sound, delay a bit longer
         if(playing_sound == MIDDLE || playing_sound == BEGIN)
            Thread.sleep(LONG_DELAY);
         else
            Thread.sleep(SHORT_DELAY);

      } catch(InterruptedException e) {}

      //Starting to play a long sound
      playing_sound = BEGIN;
   }

   /**
   * Plays the audio clip specified in the media list by ID
   * @param ID the named index indicating the sound to be played
   */
   private void playClip(int ID)
   {
      ((AudioClip)(media_list[ID].file)).play();
   }

   /**
   * Returns the list of media that this game sound control would
   *  like loaded.
   * @return All sound that GameControl would like loaded as an
   *            array of MediaRecords
   */
   public MediaRecord[] getMediaList()
   {
      return media_list;
   }
}
