package taxman;

import java.awt.*;
import java.awt.event.*;

/**
* This panel serves as a location for placement of block numbers.
* It lays out everything in a grid, where the number of rows can
* be specified but not the number of columns.  It can also handle
*  gameUpdate events which allows it to clear itself and repaint
*  on demand.
*
* There is one option for mouse event handling.  If mouse
* event handling is enabled, the panel will remove the number
*  that is selected and fire a numberChosen event.
*
* @author Jacob Crossman
* @version 1.0
*/
public class GameTable extends Panel implements GameUpdate
{
   public  static final Color  DEF_NUM_COLOR    = Color.white;
   public  static final Color  DEF_BACK_COLOR   = Color.blue;
   private static final double COL_FACTOR       = 1.0;     //1.0 cols per row

   private static final int    H_GAP            = 0;
   private static final int    V_GAP            = 0;

           GUINumber[]   numbers    = null;                 //list of numbers in the table
   private int           next_loc   = 0;                    //next location of number to be added
   private Color         border_clr = DEF_NUM_COLOR;        //Color of surrounding border

   /**
   * Does nothing
   */
   public GameTable()
   {
   }

   /**
   * This is the common update method for all GameUpdate objects.
   *  It's job is to keep the board synchronized with game state.
   *  In this case, gameUpdate only handles the NEW_GAME flag.  This
   *  method should be called first by any inherited classes.
   * @param b The current game board
   * @param m The current minimax object
   * @param flag a flag indicating the game state
   */
   public void gameUpdate(Board b, Minimax m, int flag)
   {
      if(flag == Game.NEW_GAME)
      {
         createTable(b);
      }
   }

   /**
   * Adds a number to the grid.  Inherited classes can use
   *  this method to easily add numbers to the grid
   * @param gn The number (GUINumber) to add to the grid
   */
   public void addNumber(GUINumber gn)
   {
      //Change the visible state and set the correct number
      numbers[next_loc].setNumber(gn.getNumber());
      numbers[next_loc].setBackColor(gn.getBackColor());
      numbers[next_loc].setForeColor(gn.getForeColor());
      numbers[next_loc].setBackgroundImage(gn.getBackgroundImage());
      numbers[next_loc].enableSelection();

      numbers[next_loc].repaint();
      next_loc++;
   }

   /**
   * This method will remove the given number from the grid.
   *  It is removed from the array of numbers as well as the layout.
   * @param n  The number to remove
   */
   public void removeNumber(int n)
   {
      int i;

      //Do a linear search for the first matching number
      // And when found, remove from layout and the array
      for(i = 0; i < numbers.length; i++)
         if(numbers[i].getNumber() == n)
            break;

      if(i < numbers.length)
      {
         //This will make the number invisible and
         // keep it from participating in any mouse clicks
         numbers[i].setNumber(0);
         numbers[i].setBackColor(getBackground());
         numbers[i].setForeColor(getBackground());
         numbers[i].setBackgroundImage(null);
         numbers[i].disableSelection();
         numbers[i].repaint();
      }
   }

   /**
   * This is the standard update function for components.  it is overwritten
   *  to paint the sub-canvases inside the panel
   * @param g The graphics context on which to draw
   */
   public void update(Graphics g)
   {
      //Because Netscape has such a great browser
      // I have to repaint manually!!
      for(int i = 0; i < numbers.length; i++)
         numbers[i].repaint();
   }


   /**
   * Creates the layout for this grid and enforces it by filling
   *  it with invisible '0's which are overwritten by the real
   *  numbers later.
   * @param b The board representing the new game
   */
   public void setNewGameLayout(Board b)
   {
      //Do a calculation
      int cols = (int)Math.floor(Math.sqrt(b.getNumberOfAvailableMoves()*COL_FACTOR));
      int rows = (int)Math.ceil(b.getNumberOfAvailableMoves() / cols);

      setLayout(new GridLayout(rows, cols, H_GAP, V_GAP));

      for(int i = 0; i < b.getNumberOfAvailableMoves(); i++)
      {
         GUINumber tmp = new GUINumber(0, getBackground(), getBackground());

         //Add invisible numbers to enforce the grid layout
         add(tmp);
         numbers[i] = tmp;
      }

      //Cause new layout to be displayed
      validate();
   }

   /**
   * Each grid uses the same method to create a new grid.
   *  This can be used by any inherited classes.
   * @param b The board which gives information needed in creation
   */
   public void createTable(Board b)
   {
      numbers      = new GUINumber[b.getNumberOfAvailableMoves()];      //create new array
      next_loc     = 0;                                                 //starting index new

      removeAll();
      setNewGameLayout(b);
   }
}

