package taxman;

import java.applet.*;
import java.awt.*;
import java.net.*;

/**	
* This is the loader thread for the Media Loader Design Set.
* This design set allows a program to download media
*  features as a separate thread while recieving updates
*  on the progress periodically.
*
* @Author:	Jacob Crossman
* @Version:	1
*/
public class MediaLoader implements Runnable
{
   private Applet           app       = null;            //The applet is optional for use in downloading
                                                         // images and a MUST for downloading Audio
   private MediaObserver[]  call_back = null;            //Call back object to recieve updates
   private MediaRecord[]    media     = null;            //Media Record list (all media to be loaded)
   private Component        cmp       = new Panel();     //Needed to allow download of images

   /**
   * This constructor is useful non-multithreading IMAGE loads from the local machine.
   * For added functionality, use other constructors.   */
   public MediaLoader( )
   {
   }

   /**
   * This constructor is useful non-multithreading loads.  It takes
   * an applet which is used to help load audio files or files across a network.
   * @param a The Applet to use for loading functionality and URL of the program 
   */
   public MediaLoader( Applet a )
   {
      app = a;
   }

   /**
   * This constructor is useful for multi-threading.
   * It takes a list of media items to load, a list of MediaObservers to be
   * notified of loads. This version can load ONLY IMAGES.  To load audio also
   * use the constructor that takes an applet as thelast parameter,
   * it will then be able to run and will update the user as to its progress
   * through the observers.
   * @param mr Array of media to load (can be length one,... though using the
   *            non-threaded version of getMediaItem would be more likely
   * @param mo Array of media observers which would like to be notified of
   *            media load progress
   */
   public MediaLoader( MediaRecord[] mr, MediaObserver[] mo )
   {
      call_back = mo;
      media     = mr;
   }

   /**
   * This constructor is useful for multi-threading.
   * It takes a list of media items to load, a list of MediaObservers to be
   * notify of loads and an applet for loading.  If created within a thread,
   * it will then be able to run and will update the user as to its progress
   * through the observers.
   * @param mr Array of media to load (can be length one,... though using the
   *            non-threaded version of getMediaItem would be more likely
   * @param mo Array of media observers which would like to be notified of
   *            media load progress
   * @param a  The applet that provides the loading functionality
   */
   public MediaLoader( MediaRecord[] mr, MediaObserver[] mo, Applet a )
   {
      call_back = mo;
      media     = mr;
      app       = a;
   }

   /**
   * Retrieves the media item given by the media record.  Supply the
   *  fileName and type of the media record and the method will return
   *  the completely filled record.
   * @param mr The media record describing which media to get
   * @exception MalformedURLException thrown when the URL (filename) for the item
   *               is bad or wrong
   */
   public MediaRecord getMediaItem(MediaRecord mr) throws MalformedURLException
   {
      //Choose the media to download and do-it!!
      if(mr.type == MediaRecord.AUDIO)
         mr.file = retrieveAudio(mr.fileName);
      else if(mr.type == MediaRecord.GRAPHIC)
         mr.file = retrieveImage(mr.fileName);

      //Return the filled media item
      return mr;
   }

   /**
   * Retrieves a single image given by the current document base and
   *  the path and file name in Name.  The method waits for full image
   *  load before returning.
   * @param Name path and file name of the image to load
   * @exception MalformedURLException thrown when the URL (filename) for the item
   *               is bad or wrong
   */
   public Image  retrieveImage(String Name) throws MalformedURLException
   {
      //The image must be stored temporarily cause we
      // will wait for a download
      Image tmp;

      if(app == null)  //Gotta use the Toolkit fnc
         tmp = Toolkit.getDefaultToolkit().getImage(Name);   
      else             //Applet has built-in fnc
      {
         try
         {
            //Note: if the given applet is being run in a frame, the
            // getDocumentBase won't work and will through a null pointer
            // exception.
            tmp = app.getImage(new URL(app.getDocumentBase(), Name));
         }
         catch (NullPointerException e)
         {
            //Last load attempt goes for a local version
            tmp = Toolkit.getDefaultToolkit().getImage(Name);   
         }
      }

      //Create a media tracker to garantee full download
      // before a display
      MediaTracker mt = new MediaTracker(cmp);
      try
      {
         mt.addImage(tmp, 0);
         mt.waitForID(0);
      }
      catch (InterruptedException e) { e.printStackTrace(); }

      //Give 'em their image
      return tmp;
   }

   /**
   * The static version of the retrieve image, this method does the
   *  same thing as it's non-static counterpart but does not require
   *  an object instantiation to work.
   * @param Name the path and file name for the image to load
   * @param app The applet (could be null) that is connected with the image
   * @param cmp A component (could also be the applet) that is interested
   *               in the image.
   * @exception MalformedURLException thrown when the URL (filename) for the item
   *               is bad or wrong
   */
   public static Image retrieveImage(String Name, Applet app, Component cmp) throws MalformedURLException
   {
      //The image must be stored temporarily cause we
      // will wait for a download
      Image tmp;

      if(app == null)  //Gotta use the Toolkit fnc
         tmp = Toolkit.getDefaultToolkit().getImage(Name);   
      else             //Applet has built-in fnc
      {
         try
         {
            //Note: if the given applet is being run in a frame, the
            // getDocumentBase won't work and will through a null pointer
            // exception.
            tmp = app.getImage(new URL(app.getDocumentBase(), Name));
         }
         catch (NullPointerException e)
         {
            //Last load attempt goes for a local version
            tmp = Toolkit.getDefaultToolkit().getImage(Name);   
         }
      }

      //Create a media tracker to garantee full download
      // before a display
      MediaTracker mt = new MediaTracker(cmp);
      try
      {
         mt.addImage(tmp, 0);
         mt.waitForID(0);
      }
      catch (InterruptedException e) { e.printStackTrace(); }

      //Give 'em their image
      return tmp;
   }

   /**
   * Retrieves an audio clip from the current document base using
   *  the path and name given in Name.  Note: this method will return
   *  an null handle if a constructor without an applet was used.
   * @param Name path and file name for the AudioClip to load.
   * @exception MalformedURLException thrown when the URL (filename) for the item
   *               is bad or wrong
   */
   public AudioClip retrieveAudio(String Name) throws MalformedURLException
   {
      //The applet holds the getAudioClip function we need
      if(app == null)   
         return null;

      return app.getAudioClip(new URL(app.getDocumentBase(), Name));
   }


   /**
   * The static version of the non-static function of the same name.  This
   *  one does not require an object instantiation to work.
   * @param Name path and file name for the AudioClip to load.
   * @param app The name of the applet (required) that can load the audio clip
   * @exception MalformedURLException thrown when the URL (filename) for the item
   *               is bad or wrong
   */
   public static AudioClip retrieveAudio(String Name, Applet app) throws MalformedURLException
   {
      //The applet holds the getAudioClip function we need
      if(app == null)   
         return null;

      return app.getAudioClip(new URL(app.getDocumentBase(), Name));
   }

   /**
   * This method runs through the list of observers and
   * notifies them that one particular media item has been
   * fully loaded.
   * @param mr The media record that was fully loaded.
   */
   private void notifySingleLoad(MediaRecord mr)
   {
      //Notify all observers that a given media
      // item has been downloaded
      for(int i = 0; i < call_back.length; i++)
         call_back[i].itemLoaded(mr);
   }

   /**
   * Notifies all observers that the entire set of media
   *  items was loaded.
   * @param mr An array of media items that was loaded
   */
   private void notifyCompleteLoad(MediaRecord[] mr)
   {
      //Notify observers that the entire set
      // of media items has been downloaded
      for(int i = 0; i < call_back.length; i++)
         call_back[i].loadingFinished(mr);
   }

   /**
   * For multi-threaded apps, this method allow the load process
   *  to procede as a separate  thread.  Observers are notified
   *  as to the progress of the download from time to time.
   */
   public void run()
   {
      //Multi-threading only works if there is a valid media record
      // and if there are some call back objects to notify of the results
      // of the load
      if(media != null && call_back != null)
      {
         //Load each media item and notify the observers
         for(int i = 0; i < media.length; i++)
         {
            try
            {
               notifySingleLoad(media[i] = getMediaItem(media[i]));
            } catch (MalformedURLException e) {}
         }

         //Done loading, notify the observers
         notifyCompleteLoad(media);
      }
   }

}
