package taxman;

/**
* This is the implementation of Minimax with alpha/beta pruning.
*  One downside to Minimax is its massive usage of space and stack.
*  These two problems are helped by the use of undo which allows
*  the same program state structure (board, etc) to be used
*  for all tests.  Since only one move is evaluated at a time,
*  the program can effectively undo moves to find the best move.
*/
public class Minimax implements Runnable
{
   /** Infinity value for alpha/beta cutoff */
   private static final int  INFINITY     = 1000000000;
   private static final long WAIT_TIME    = 1000;
   private static final long MAX_HOG_TIME = 300; //INFINITY;    //200
   private static final long SLEEP_TIME   = 100;
   public  static final int  BLOCK_WAIT_TIME = 1000;

   private Board     board;               //The board to evaluate
   private int       ply;                 //how deep to search
   private int       depth;               //depth of search so far
   private int       perc_adv;            //Computer's percieved advantage
   private boolean   doCutoffs;           //true if alpha/beta pruning should be used

   /** Statistical variable (number of condition evaluations */
   private int       evaluations;
   /** Statistical variable (number of alpha/beta cutoffs */
   private int       cutoffs;
   /** Statistical variable telling the maximum depth travelled to */
   private int       max_depth;

   private Game      call_back = null;
   private long      hog_time  = 0;
   private boolean   interrupted = false;
   private boolean   isBlocked = false;


   /**
   * Creates a minimax object with the given board and ply (search depth).
   * @param B The board object to use for search
   * @param Ply The depth of the search (can be changed)
   * @param DoCutoffs True if the algorithm should use alpha/beta pruning
   * @param cb Call back function to use when algorithm finds a valid number
   */
   public Minimax(Board B, int Ply, boolean DoCutoffs, Game cb)
   {
      board = B;
      ply   = Ply;
      doCutoffs = DoCutoffs;
      call_back = cb;
   }

   /**
   * Sets the search ply (depth) of the algorithm
   * @param Ply The ply of the search (depth)
   */
   public final synchronized void setPly(int Ply) { ply = Ply; }

   /**
   * Gets the last player to evaluate's percieved advantage.
   *  A positive number means it is believed the player will win.
   *  A negative number means it is believed the player will lose.
   * @return The percieved advantage of the last player to evaluate
   */
   public final int getPercievedAdvantage() { return perc_adv; }

   /**
   * Gets the number of conditional checks made on evaluations.
   * @return The number of conditional evaluation checks made
   */
   public final int getEvaluations() { return evaluations; }

   /**
   * Gets the number of cutoffs made during last search.
   * @return The number of alpha/beta cutoffs made
   */
   public final int getCutoffs() { return cutoffs; }

   /**
   * Returns the current ply (depth) used in minimax search
   * @return The current ply (depth) of searching
   */
   public final int  getPly() { return ply; }

   /**
   * Returns the maximum depth traveled to in this evaluation set
   * @return The maximum depth in this search
   */
   public final int getMaxDepth() { return max_depth; }

   /**
   * Finds and returns the best move within the current ply.
   *  The best move is given w.r.t the current player.
   * @return An object representing the best move
   */
   public final synchronized Object findBestMove()
   {
      depth       = -1;    //Initialize search depth to 0
      max_depth   = -1;    //Stats
      evaluations = 0;     //Stats
      cutoffs     = 0;     //Stats

      prepareThread();     //prepares the current thread for the run

      MoveSet tmp = doMinimax(-INFINITY, INFINITY);
      perc_adv    = tmp.value;

      return tmp.move;
   }

   /**
   * Causes the algorithm to be halted and the thread
   *  running the algorithm to exit.
   */
   public final void quit()
   {
      interrupted = true;
   }

   /**
   * Prepares the thread running this algorithm for
   *  excecution.
   */
   private final synchronized void prepareThread()
   {
      interrupted = false;

      //Lower thread priority so it will not
      // hog the CPU
      Thread tmp  = Thread.currentThread();
      tmp.setPriority(tmp.getPriority() - 1);
   }

   /**
   * Checks the current clock and if a certain amount of time
   * has passsed, it delays the thread for a defined period
   */
   private final synchronized void delay()
   {
      //Prevents the minimax algorithm from taking all the time
      // from the CPU in an applet
      if(System.currentTimeMillis() > hog_time)
      {
         try
         {
            Thread.sleep(SLEEP_TIME);
            hog_time = System.currentTimeMillis() + MAX_HOG_TIME;
         } catch (InterruptedException e) {}
      }
   }

   /**
   * Does a recursive minimax search with alpha/beta pruning.
   *  What is returned is the best move from that point below in
   *  the move tree.  By the end, the best move (and it's value)
   *  is returned.
   * @param alpha The current alpha value used for cutoff
   * @param beta The current beta value used for cutoff
   * @return The best move to this point
   */
   private final synchronized MoveSet doMinimax(int alpha, int beta)
   {
      //Delay if the algorithm is hogging the clock time
      delay();

      depth++;

      //Depth reached or there are no more moves available (evaluate where
      // it stands)
      if(depth >= ply || board.getNumberOfAvailableMoves() == 0)
      {
         if(depth > max_depth)
            max_depth = depth;

         depth--;
         return new MoveSet(board.evaluate(), null);
      }
      else
      {
         MoveSet  temp = null;
         MoveSet  best = new MoveSet(alpha, null);

         //Loop through all possible moves.
         // Short circuit evaluation should prevent temp.value from throwing
         // a null pointer exception.
         for(int i = 0, n = -1; i < board.getNumberOfAvailableMoves(); i++)
         {
            //This will cause a NullPointerException to be thrown
            // which exits the algorithm
            if(interrupted)
               return null;

            //Find the index of the next possible move
            n = board.getNextIndexedMove(n);

            board.makeIndexedMove(n);                          //make the ith possible move
            temp = doMinimax(-beta, -best.value);              //evaluate
            board.undoLastMove();                              //undo the move

            evaluations++;

            if((-temp.value) > best.value)            //is it better?
            {
               best.value = -temp.value;
               best.move  = board.getMoveAtIndex(n);
            }

            //Beta or Alpha cutoff (it switches based on particular
            // function call). Move evaluations won't help, stop.
            if(best.value >= beta && doCutoffs)
            {
               cutoffs++;
               break;
            }
         }

         depth--;
         return best;
      }
   }

   /**
   * Runs algorithm and reports back to the call-back object.
   */
   public void run()
   {
      //Start an infinite loop that ends when stop is called
      //  by the creating function
      long time = System.currentTimeMillis() + WAIT_TIME;
      hog_time  = time;

      try
      {
         //Does the dirty work
         TaxNumber t = (TaxNumber)findBestMove();

         //Force a wait so user gets impression the computer is thinking
         if(System.currentTimeMillis() < time)
         {
            //Sleep for remaining wait time
            Thread.sleep(Math.abs(time - System.currentTimeMillis()));
         }

         //Get back to processing so the human can choose
         call_back.makeCPUMove(t.getValue());
      }
      catch (InterruptedException e) {}
      catch (NullPointerException e) {}
   }

   /** Minimax algorithm tester */
   public static void main(String[] args)
   {
      Board board = new Board();
      Minimax minimax = new Minimax(board, 10, true, new Game());

      for(int i = 13; i > 0; i--)
         board.addNumber(i);

      board.endBoardFill();

      board.displayState();

      /**
      while (board.getNumberOfAvailableMoves() > 0)
      {
      */
         TaxNumber t = (TaxNumber)minimax.findBestMove();
         System.out.println("Best Move: " + t);
         System.out.println("Depth: " + minimax.getMaxDepth());
         System.out.println("Evals: " + minimax.getEvaluations());
         System.out.println("Cutoffs: " + minimax.getCutoffs());

         board.makeMove(t);
         board.displayState();
      //}
   }
}

/**
* Little class to hold move temporarily.
*  It should be INSIDE the Minimax but
*  due to Java 1.0 restrictions,...
*/
class MoveSet {
   Object    move;                     //the move to make (index)
   int       value;                    //the value of the move

   public MoveSet(int V, Object O) { value = V; move = O; }
};
