package taxman;
import  java.io.*;
import  java.util.*;

/**
* This class is the main module for the Taxman game.
* It is designed to allow interactive play between the cpu
*  and a person.  This particular class focuses on outputting
*  stats so that the results can be used in a report.
*/
public class Game extends Object
{
   private static final int      DEF_NUMS_USED  = 15;
   private static final boolean  DEF_RANDOM     = true;
   private static final boolean  DEF_CPU_FIRST  = false;
   private static final int      DEF_LOOK_AHEAD = 4;
   private        final boolean  USE_CUTOFF     = true;

   private Board   board;
   private Minimax minimax;

   private int     nums_used  = DEF_NUMS_USED;
   private boolean random     = DEF_RANDOM;
   private boolean cpu_first  = DEF_CPU_FIRST;
   private int     look_ahead = DEF_LOOK_AHEAD;

   public Game() {}

   /**
   * Gets user input on several game parameters.
   */
   public void getGameParams()
   {
      String userResponse;

      //Set up to read data
      BufferedReader ds = new BufferedReader( new InputStreamReader( System.in ), 1 );

      System.out.println("++ Game Setup ++++++++++++++++++++++++++++++++");

      /////////////////////////////////////////////////////////////////////
      System.out.print("How many numbers should be in the game [" + DEF_NUMS_USED + "]: ");

      try
      {
         userResponse = ds.readLine();
         nums_used    = Integer.parseInt(userResponse);

      } catch (Exception e) { }

      if(nums_used <= 0)
         nums_used = DEF_NUMS_USED;


      /////////////////////////////////////////////////////////////////////
      System.out.print("Use random numbers? Type 'y' or 'n' [" +
                       ((DEF_RANDOM)? "y": "n") + "]: ");

      try
      {
         userResponse = ds.readLine();
         random       = (userResponse.charAt(0) == 'y')? true: false;

      } catch (Exception e) { }


      /////////////////////////////////////////////////////////////////////
      System.out.print("Do you want to go first? Type 'y' or 'n' [" +
                       ((DEF_CPU_FIRST)? "n": "y") + "]: ");

      try
      {
         userResponse = ds.readLine();
         cpu_first    = (userResponse.charAt(0) == 'n')? true: false;

      } catch (Exception e) { }


      /////////////////////////////////////////////////////////////////////
      System.out.print("How many levels ahead should the computer think? [" + DEF_LOOK_AHEAD + "]: ");

      try
      {
         userResponse = ds.readLine();
         look_ahead   = Integer.parseInt(userResponse);

      } catch (Exception e) { }

      if(look_ahead <= 0)
         look_ahead = DEF_LOOK_AHEAD;

      //Repeat what was typed for verification of the user
      System.out.println("\n\nNumber of numbers: " + nums_used);
      System.out.println("Use random numbers: " + random);
      System.out.println("HUMAN first: " + !cpu_first);
      System.out.println("Levels of thought: " + look_ahead);
      System.out.println("\n\n");
   }

   /**
   * Loads the board with random numbers.
   * Numbers are between 1 and the max numbers in board.
   * Duplicates are allowed.
   */
   public void loadRandom()
   {
      Random r = new Random(System.currentTimeMillis());

      System.out.println("\nNumbers Used: " + nums_used);

      for(int i = 0; i < nums_used; i++)
      {
         double tmp = Math.random() * nums_used * 2;
         board.addNumber((int)Math.round(tmp));
      }

   }

   /**
   * Loads the board with sequential numbers up to the number
   * of numbers in the board.
   */
   public void loadInOrder()
   {
      for(int i = 1; i <= nums_used; i++)
         board.addNumber(i);
   }

   /**
   * Sets up the board for playing.
   */
   public void setUpBoard()
   {
      board    = new Board(((cpu_first)? Board.CPU: Board.HUMAN));
      minimax  = new Minimax(board, look_ahead, USE_CUTOFF);

      if(random)
         loadRandom();
      else
         loadInOrder();
   }

   /**
   * Reads in human move.  Invalid moves are ignored.
   */
   public void doHumanMove()
   {
      if(board.getNumberOfAvailableMoves() > 0)
      {

         String userResponse;

         //Set up to read data
         BufferedReader ds = new BufferedReader( new InputStreamReader( System.in ), 1 );

         boolean invalidInput = true;

         board.displayState();
         while(invalidInput)
         {
            System.out.print("\nType the number you wish to choose: ");

            try
            {
               userResponse = ds.readLine();
               if(!board.makeMove(Integer.parseInt(userResponse)))
                  continue;
            } catch(Exception e) { continue; }

            invalidInput = false;
         }
      }
   }

   /**
   * Chooses the best move for the CPU then displays the board state.
   */
   public void doCPUMove()
   {
      if(board.getNumberOfAvailableMoves() > 0)
      {
         System.out.println("\n\n\n\n\n\n\n\n\n\n\n\n");
         System.out.println("====================================================");
         System.out.println("Computer Thinking,... Please Wait");

         board.makeMove((TaxNumber)minimax.findBestMove());

         System.out.println("Evaluations: " + minimax.getEvaluations());
         System.out.println("Cutoffs: " + minimax.getCutoffs());
         System.out.println("Max Depth: " + minimax.getMaxDepth());
         System.out.println("CPU Advantage: " + minimax.getPercievedAdvantage());
         System.out.println("");
      }
   }

   /**
   * Actually starts the game playing.
   */
   public void playGame()
   {

      //Make first move if the CPU is set to do that
      if(cpu_first)
         doCPUMove();

      //Keep asking for moves as long as numbers are left to choose
      while(board.getNumberOfAvailableMoves() > 0)
      {
         doHumanMove();
         doCPUMove();
      }

      /*
      while(board.getNumberOfAvailableMoves() > 0)
      {
         board.displayState();
         doCPUMove();
      }
      */

      //Show final state
      System.out.println("\n==================================================");
      board.displayState();

      //Show special message
      System.out.println("== GAME OVER == GAME OVER == GAME OVER ==");
      System.out.println("== You " + ((board.getWinner() == board.CPU)? "LOST, BUMMER!!": "WON, WOW!!"));
      System.out.println("== FINAL SCORE == YOU: " + board.getHumanScore() + ";  Computer: " + board.getCPUScore());
   }


   public static void main(String[] args) throws IOException
   {
      BufferedReader br = new BufferedReader( new InputStreamReader( System.in ), 1 );

      do
      {
         Game game = new Game();

         game.getGameParams();
         game.setUpBoard();
         game.playGame();

         System.out.println("\n\n");
         System.out.print("Play another game? Type 'y' or 'n' [n]: ");

      }while(br.readLine().equals("y") || br.readLine().equals("Y"));
   }

}
