
package taxman;

import  java.awt.*;

/**
* This is the lower left button panel for the applet.
*  It controls the new game and options buttons and
*  supplies a Checkbox for the user to choose to go first.
*
*  @author Jacob Crossman
*  @version 1.0
*/
public class OptionsPanel extends Panel implements GameUpdate
{
   private static final Font  TITLE_FONT = new Font("Helvetica", Font.BOLD, 16);
   private static final Font  LABEL_FONT = new Font("Helvetica", Font.BOLD, 12);
   private static final Color FORE_COLOR = Color.black;
   private static final Color BACK_COLOR = Color.white;

   //Static field labels and the title of the options screen
   private Label     labTitle   = new Label("Taxman Options");
   private Label     labLevels  = new Label("Search Depth: ");
   private Label     labNumbers = new Label("Quantity of Numbers: ");
   private Label     labRandom  = new Label("Use Random Numbers: ");
   private Label     labAlpha   = new Label("Use Alpha/Beta Pruning: ");
   private Label     labSound   = new Label("Sound ON/OFF: ");

   //Dynamic fields that take user input
   private TextField txtLevels  = new TextField(4);
   private TextField txtNumbers = new TextField(4);
   private Checkbox  chkRandom  = new Checkbox();
   private Checkbox  chkAlpha   = new Checkbox();
   private Checkbox  chkSound   = new Checkbox();

   //The "OLD" values serve to hold the values on opening the
   // options so they can be returned on exit with 'CANCEL'
   private int       intLevels  = 0;
   private int       intNumbers = 0;
   private boolean   boolRandom = false;
   private boolean   boolAlpha  = false;
   private boolean   boolSound  = false;

   //"OK" button to leave options and the usual cancel
   private Button    btnOk      = new Button("OK");
   private Button    btnCancel  = new Button("Cancel");

   //Game control object which will be notified on options
   // changes and button presses.
   private Game      control    = null;

   /**
   * Initializes UI components and sets the layout.
   */
   public OptionsPanel(Game g)
   {
      control = g;

      initializeObjects();
      createLayout();
   }

   private void initializeObjects()
   {
      //Set up font for title
      labTitle.setFont(TITLE_FONT);
      labTitle.setAlignment(Label.CENTER);

      //All the field labels have same font and
      // alignment
      labLevels.setFont(LABEL_FONT);
      labLevels.setAlignment(Label.RIGHT);
      labNumbers.setFont(LABEL_FONT);
      labNumbers.setAlignment(Label.RIGHT);
      labRandom.setFont(LABEL_FONT);
      labRandom.setAlignment(Label.RIGHT);
      labAlpha.setFont(LABEL_FONT);
      labAlpha.setAlignment(Label.RIGHT);
      labSound.setFont(LABEL_FONT);
      labSound.setAlignment(Label.RIGHT);

      //Put default text/states in each field.  The defaults
      // are kept in the Game object.
      txtLevels.setText(Integer.toString(Game.DEF_LOOK_AHEAD));
      txtNumbers.setText(Integer.toString(Game.DEF_NUMS_USED));
      chkRandom.setState(Game.DEF_RANDOM);
      chkAlpha.setState(Game.DEF_USE_CUTOFF);
      chkSound.setState(Game.DEF_SOUND_STATE);
   }

   /**
   * Does the layout fancy work for the applet.
   *  It uses a gridBagLayout to create its particular look.
   */
   private void createLayout()
   {
      Panel              ctr = new Panel();

      GridBagLayout      gbl = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();

      ctr.setLayout(gbl);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(10,5,5,5);

      //Places TITLE on top
      gbc.anchor    = GridBagConstraints.CENTER;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 1;
      gbc.gridy     = 1;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      gbl.setConstraints(labTitle, gbc);
      ctr.add(labTitle);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(2,5,2,0);

      //Places Number of Search Levels second
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 1;
      gbc.gridy     = 2;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(labLevels, gbc);
      ctr.add(labLevels);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(2,0,2,5);

      //Places Search Levels modifiable field
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 2;
      gbc.gridy     = 2;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(txtLevels, gbc);
      ctr.add(txtLevels);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(2,5,2,0);

      //Places Quantity of Numbers tag
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 1;
      gbc.gridy     = 3;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(labNumbers, gbc);
      ctr.add(labNumbers);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(2,0,2,5);

      //Places Number of numbers update field
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 2;
      gbc.gridy     = 3;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(txtNumbers, gbc);
      ctr.add(txtNumbers);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(2,5,2,0);

      //Places Use Random Numbers Tag
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 1;
      gbc.gridy     = 4;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(labRandom, gbc);
      ctr.add(labRandom);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(2,0,2,5);

      //Places Use Random Numbers Checkbox
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 2;
      gbc.gridy     = 4;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(chkRandom, gbc);
      ctr.add(chkRandom);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(2,5,2,0);

      //Places Alpha/Beta Cutoff Tag
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 1;
      gbc.gridy     = 5;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(labAlpha, gbc);
      ctr.add(labAlpha);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(2,0,2,5);

      //Places Alpha/Beta Cutoff Checkbox field
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 2;
      gbc.gridy     = 5;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(chkAlpha, gbc);
      ctr.add(chkAlpha);

      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(2,5,2,0);

      //Places Sound description Tag
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 1;
      gbc.gridy     = 6;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(labSound, gbc);
      ctr.add(labSound);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(2,0,2,5);

      //Places Sound Checkbox field
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 2;
      gbc.gridy     = 6;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(chkSound, gbc);
      ctr.add(chkSound);

      ////////////////////////////////////////////////
      //Unfortunately, yet another panel is needed
      // to layout the buttons in an orderly manner
      Panel button_panel = new Panel();
      GridBagLayout gbl2 = new GridBagLayout();

      button_panel.setLayout(gbl2);

      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(0,10,0,10);

      //Places Ok Button
      gbc.anchor    = GridBagConstraints.CENTER;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 1;
      gbc.gridy     = 1;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbl2.setConstraints(btnOk, gbc);
      button_panel.add(btnOk);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(0,10,0,10);

      //Places Cancel Button
      gbc.anchor    = GridBagConstraints.CENTER;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 2;
      gbc.gridy     = 1;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbl2.setConstraints(btnCancel, gbc);
      button_panel.add(btnCancel);

      //Now we add the button_panel to the full layout!!!

      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(5,0,5,0);

      //Places Button Panel centered at the bottom
      gbc.anchor    = GridBagConstraints.CENTER;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 1;
      gbc.gridy     = 7;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      gbl.setConstraints(button_panel, gbc);
      ctr.add(button_panel);


      //We have to add the center panel as a compenent of
      // this panel so that the options will center nicely

      GridBagLayout      gbl3 = new GridBagLayout();
      //BorderPanel        bp   = new BorderPanel(ctr, 5, 5, 5, 5, Color.black);

      setLayout(gbl3);

      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(0,0,10,0);

      //Places All Options in the middle
      gbc.anchor    = GridBagConstraints.CENTER;
      gbc.fill      = GridBagConstraints.NONE;
      gbc.gridx     = 1;
      gbc.gridy     = 1;
      gbc.weightx   = 0.0;
      gbc.weighty   = 0.0;
      gbl3.setConstraints(ctr, gbc);
      add(ctr);

      validate();
   }

   /**
   * Over-ride of the inherited function to keep the
   *  text box and button text from changing color (they won't be clear)
   * @param clr The new foreground color
   */
   public void setBackground(Color clr)
   {
      super.setBackground(clr);

      //propagate background change to sub-components
      labTitle.setBackground(clr) ;
      labLevels.setBackground(clr);
      labNumbers.setBackground(clr);
      labRandom.setBackground(clr);
      labAlpha.setBackground(clr);
      labSound.setBackground(clr);
      chkRandom.setBackground(clr);
      chkAlpha.setBackground(clr);
      chkSound.setBackground(clr);

      //keep same
      txtLevels.setBackground(BACK_COLOR);
      txtNumbers.setBackground(BACK_COLOR);
      btnOk.setBackground(Color.lightGray);
      btnCancel.setBackground(Color.lightGray);
   }

   /**
   * Over-ride of the inherited function to keep the
   *  text box and button text from changing color (they won't be clear)
   * @param clr The new background color
   */
   public void setForeground(Color clr)
   {
      super.setForeground(clr);


      //propagate forground change to sub-components
      labTitle.setForeground(clr) ;
      labLevels.setForeground(clr);
      labNumbers.setForeground(clr);
      labRandom.setForeground(clr);
      labAlpha.setForeground(clr);
      labSound.setForeground(clr);
      chkRandom.setForeground(clr);
      chkAlpha.setForeground(clr);
      chkSound.setForeground(clr);

      //Reverse Video
      txtLevels.setForeground(FORE_COLOR);
      txtNumbers.setForeground(FORE_COLOR);
      btnOk.setForeground(FORE_COLOR);
      btnCancel.setForeground(FORE_COLOR);
   }


   /**
   * The standard update method from GameUpdate
   * This one handles OPTIONS_OPEN.
   * @param b The current game board
   * @param m The current minimax object
   * @param flags Indicator of current state
   */
   public void gameUpdate(Board b, Minimax m, int flags)
   {
      if(flags == Game.OPTIONS_OPEN)
         setOldValues();
   }

   /**
   * Only thing to look for is button presses from
   *  the ok button and the cancel button.
   * @param evt The actual event that occured
   * @param obj The object that was effected by the event
   */
   public boolean action(Event evt, Object obj)
   {
      if(evt.target == btnOk)
         okButtonPressed();
      else if(evt.target == btnCancel)
         cancelButtonPressed();

      return true;
   }

   /**
   * Handles OK button presses.  It grabs values from the updated fields,
   *  verifies the values (as much as it can), and updates the game.
   *  Finally, it exits the options panel
   */
   private void okButtonPressed()
   {
      //Store temporary values from fields so that they
      // can be checked
      setOldValues();

      if(intLevels <= 0)
         intLevels = 1;

      if(intNumbers <= 0)
         intNumbers = 1;

      //Tell game control that the options for the game have changed
      control.optionsChanged(boolAlpha, intLevels, intNumbers, boolRandom, boolSound);

      //Cancel will put the temporary values in and close the
      // options panel
      cancelButtonPressed();
   }

   /**
   * The Cancel button press handler.  It returns temporary values
   *  to each field and exits the options panel.
   */
   private void cancelButtonPressed()
   {
      returnOldValues();
      control.changeOptionsScreen(false);
   }

   /**
   * Sets the temporary values for each field before the
   *  options become available on screen. These temporary
   *  values can be written back when the user presses cancel.
   */
   private void setOldValues()
   {
      //Save all field values
      intLevels  = Integer.parseInt(txtLevels.getText());
      intNumbers = Integer.parseInt(txtNumbers.getText());
      boolRandom = chkRandom.getState();
      boolAlpha  = chkAlpha.getState();
      boolSound  = chkSound.getState();
   }

   /**
   * Returns the old temporary values that were set
   *  when the options opened.  This way, the user
   *  can press Cancel and really cancel their actions.
   */
   private void returnOldValues()
   {
      //Return all values to the field components
      txtLevels.setText(Integer.toString(intLevels));
      txtNumbers.setText(Integer.toString(intNumbers));
      chkRandom.setState(boolRandom);
      chkAlpha.setState(boolAlpha);
      chkSound.setState(boolSound);
   }
}
