
package taxman;

import  java.awt.*;

/**
* This is a special canvas that acts as a status bar.
*  You can set the insets (space on all sides), update
*  its value and set a max value for it. 
*
*  @author Jacob Crossman
*  @version 1.0
*/
public class ProgressBar extends Canvas
{
   private boolean allow_negative_nums = false;
   private boolean show_numbers        = true;
   private boolean addPercent          = false;

   private float  cur_value            = 0;
   private float  max_value            = 1;

   private Insets  spacing             = new Insets(0,0,0,0);

   /**
   * Simplest constructor. Uses all defaults
   * Positive numbers, no numeric input, initial value = 0, max value = 1
   */
   public ProgressBar() {}
   
   /**
   * Lets you supply the intitial value and maximum value. (floats)
   * @param init_val The initial value of the progress bar
   * @param max_val  The maximum value of the progress bar
   */ 
   public ProgressBar(float init_val, float max_val)
   {
      cur_value = init_val;
      max_value = max_val;
   }

   /**
   * Lets you supply the intitial value and maximum value. (ints)
   * @param init_val The initial value of the progress bar
   * @param max_val  The maximum value of the progress bar
   */ 
   public ProgressBar(int init_val, int max_val)
   {
      cur_value = (float)init_val;
      max_value = (float)max_val;
   }

   /**
   * Lets you supply the intitial value and maximum value. (floats)
   * @param init_val The initial value of the progress bar
   * @param max_val  The maximum value of the progress bar
   * @param AddPercent True if the progress bar should add a percent sign
   */ 
   public ProgressBar(float init_val, float max_val, boolean AddPercent)
   {
      cur_value  = init_val;
      max_value  = max_val;
      addPercent = AddPercent;
   }

   /**
   * Lets you supply the intitial value and maximum value. (ints)
   * @param init_val The initial value of the progress bar
   * @param max_val  The maximum value of the progress bar
   * @param AddPercent True if the progress bar should add a percent sign
   */ 
   public ProgressBar(int init_val, int max_val, boolean AddPercent)
   {
      cur_value  = (float)init_val;
      max_value  = (float)max_val;
      addPercent = AddPercent;
   }

   /**
   * Lets you supply the intitial value and maximum value. (floats)
   *  It also lets you specify whether or not to use a number value on the bar (i.e. %)
   *  and whether or not to allow negative values
   * @param init_val The initial value of the progress bar
   * @param max_val  The maximum value of the progress bar
   * @param Negative True if negative values are allowed (bar will be centered)
   * @param ShowNums True if a number should be written into middle of bar.
   */ 
   public ProgressBar(float init_val, float max_val, boolean Negative, boolean ShowNums)
   {
      cur_value = init_val;
      max_value = max_val;

      allow_negative_nums = Negative;
      show_numbers        = ShowNums;
   }

   /**
   * Lets you supply the intitial value and maximum value. (ints)
   *  It also lets you specify whether or not to use a number value on the bar (i.e. %)
   *  and whether or not to allow negative values
   * @param init_val The initial value of the progress bar
   * @param max_val  The maximum value of the progress bar
   * @param Negative True if negative values are allowed (bar will be centered)
   * @param ShowNums True if a number should be written into middle of bar.
   */ 
   public ProgressBar(int init_val, int max_val, boolean Negative, boolean ShowNums)
   {
      cur_value = (float)init_val;
      max_value = (float)max_val;

      allow_negative_nums = Negative;
      show_numbers        = ShowNums;
   }

   /**
   * Sets the value of the progress bar to some number < maximum value
   * @param NewValue The new value for the progress bar
   */
   public void setValue(int NewValue)
   {
      cur_value = (float)NewValue;
   }

   /**
   * Sets the value of the progress bar to some number < maximum value
   * @param NewValue The new value for the progress bar
   */
   public void setValue(float NewValue)
   {
      cur_value = NewValue;
   }

   /**
   * Gets the current value of the progress bar as a float
   * @return Returns a float version of the progress bar value
   */
   public float getFloatValue() 
   {
      return cur_value;
   }

   /**
   * Gets the current value of the progress bar as an int
   * @return Returns an int version of the progress bar value
   */
   public float getIntValue()
   {
      return Math.round(cur_value);
   }

   /**
   * Sets the maximum value for this progress bar
   * @param max The maximum value for this progress bar
   */
   public void setMaxValue(int max)
   {
      max_value = (float)max;
   }

   /**
   * Gets the current value of the progress bar as an int
   * @return Returns an int version of the progress bar value
   */
   public void setMaxValue(float max)
   {
      max_value = max;
   }

   /**
   * Gets the maximum value of the progress bar as a float
   * @return Returns a float version of the progress bar maximum value
   */
   public float getMaxFloatValue()
   {
      return max_value;
   }


   /**
   * Gets the maximum value of the progress bar as an int
   * @return Returns an int version of the progress bar maximum value
   */
   public int getMaxIntValue()
   {
      return (int)Math.round(max_value);
   }

   /**
   * Sets the spacing around the status bar
   * @param NewInsets An Insets object indicating the right, top, left bottom spacing
   *   around the status bar.
   */
   public void setInsets(Insets NewInsets)
   {
      spacing = NewInsets;
   }

   /**
   * Gets the current value of the spacing surrounding the progress bar
   * @return An Insets object indicating the right, top, left bottom spacing
   *   around the status bar.
   */
   public Insets getInsets()
   {
      return spacing;
   }

   /**
   * Paints the progress bar
   * @param g The graphics context on which to draw
   */
   public void paint(Graphics g)
   {
      if(allow_negative_nums)
         drawFromCenter(g);
      else
         drawFromLeft(g);

      if(show_numbers)
         drawValue(g);
   }

   /**
   * Draws a progress bar from the center of the screen.
   * @param g the graphics context on which to draw
   */
   private void drawFromCenter(Graphics g)
   {
      float    progress = 0;                     //number represnting pixel length of bar
      Rectangle r        = bounds();              //bounds of the canvas


      //Overwrite previous background
      g.setColor(getBackground());
      g.fillRect(r.x, r.y, r.width, r.height);

      //Set upd to paint bar
      g.setColor(getForeground());
      
      //Draw a starting location (baseline)
      g.drawRect((r.width / 2),
                 0,
                 1,
                 r.height);

      //Negative points left, positive right
      if(cur_value < 0)
      {
         progress = (-cur_value / max_value)*((r.width / 2) - spacing.left);
      
         //Draw it backwards
         g.fillRect(((r.width / 2) - (int)Math.round(progress)), 
                    spacing.top,
                    (int)Math.round(progress),
                    r.height - spacing.top - spacing.bottom);
      }
      else
      {
         progress = (cur_value / max_value)*((r.width / 2) - spacing.right);
         
         //Draw it!!
         g.fillRect((r.width / 2),
                    spacing.top,
                    (int)Math.round(progress),
                    r.height - spacing.top - spacing.bottom);
      }

   }

   /**
   * Draws the progress bar from left to right
   * @param g The graphics context on which to draw
   */
   public void drawFromLeft(Graphics g)
   {
      float    progress = 0;                     //number represnting pixel length of bar
      Rectangle r        = bounds();              //bounds of the canvas

      //Overwrite previous bar
      g.setColor(getBackground());
      g.fillRect(r.x, r.y, r.width, r.height);

      //Set up to paint new bar
      g.setColor(getForeground());


      //Draw a starting location (baseline)
      g.drawRect(spacing.left,
                 0,
                 1,
                 r.height);
      
      progress = (cur_value / max_value)*(r.width - spacing.right - spacing.left);

      //Draw it!!
      g.fillRect(spacing.left,
                 spacing.top,
                 (int)Math.round(progress),
                 r.height - spacing.top - spacing.bottom);
   }

   /**
   * Draws the value of the progress bar (numerically)
   *
   */
   public void drawValue(Graphics g)
   {
      Rectangle    r  = bounds();
      FontMetrics fm  = g.getFontMetrics();
      String      str = null;

      //If it is an integer, don't print out decimal spaces
      if(((float)Math.round(cur_value)) == cur_value)
         str = Integer.toString((int)cur_value);
      else
         str = Float.toString(cur_value);
       
      if(addPercent)
         str = str.concat("%");

      //Need halves of so many things, might as well
      // define 'em
      int half_w = Math.round((float)r.width / 2);
      int half_h = Math.round((float)r.height / 2);
      int half_str_w = Math.round((float)fm.stringWidth(str) / 2);
      int half_str_h = Math.round((float)fm.getAscent() / 2);

      g.setColor(getBackground());
      //Create a rectangle to hold the value
      g.fillRect(half_w - half_str_w,
                 half_h - half_str_h - 1,
                 2*half_str_w + 2,
                 2*half_str_h + 2);
                    
      g.setColor(getForeground());
      //Draw the actual value
      g.drawString(str,
                   half_w - half_str_w + 2,
                   half_h + half_str_h - 2);
   }
}
