package taxman;
import  java.io.*;


/**
* This little class represents a number.
* The number has a set of factors associated with it.
*  This object can return those factors and stores
*  a count of the factors
*/
public class TaxNumber extends Object
{
   private int    number;                    //The actual number (e.g. 45)
   private int[]  factors;                   //Array of factors (e.g. 1 3 5 ,...)

   public TaxNumber(int N)
   {
      number = (N > 0)? N: 1;
      setFactors();
   }

   /**
   * Sets the factor array for this TaxNumber.  The Factor array
   *  holds the list of factors of this number.
   */
   private final void setFactors()
   {
      factors = new int[countFactors()];

      int max_number = (number / 2) + 1;  //Never a factor after half of the number
      int index      = 1;
      int size       = factors.length - 1;
      int quotient;

      factors[0]    = 1;                     //always a factor
      factors[size] = number;                //number itself a factor

      //This loop quickly puts the factors in numerical
      // order in the factors array (it uses the same
      // cutoff method as the countFactos does)
      for(int i = 2; i < max_number; i++)
      {
         if((number % i) == 0)
         {
            quotient = number / i;

            factors[index] = i;                    //place low factor
            factors[size - index] = quotient;      //place high factor (opposite end)

            index++;

            max_number = quotient;
         }
      }
   }

   /**
   * Very fast factor counter.
   * @Returns the number of factors in the number
   */
   private final int countFactors()
   {
      int max_number = (number / 2) + 1;     //Never a factor after half of the number
      int count      = (number == 1)? 1: 2;  //1 & number always factors
      int quotient;

      //Find factors and adjust the upper limit
      // to shrink search time considerably
      // If a factor is found, count it and its co-factor.
      // Then, make the maximum number the co-factor to jump out
      // of the loop sooner.
      for(int i = 2; i < max_number; i++)
      {
         if((number % i) == 0)
         {
            quotient = number / i;

            count += (quotient == i)? 1: 2;
            max_number = quotient;
         }
      }

      return count;
   }

   /**
   * Returns true if the number, n, is a factor of this TaxNumber.
   * @return  true if the number, n, is a factor of this TaxNumber. false otherwise
   */
   public final boolean isAFactor(TaxNumber n)
   {
      return ((number % n.number) == 0)? true: false;
   }


   /**
   * Returns true if the number, n, is a factor of this TaxNumber.
   * @return  true if the number, n, is a factor of this TaxNumber. false otherwise
   */
   public final boolean isAFactor(int n)
   {
      return (n > 0 && (number % n) == 0)? true: false;
   }

   /**
   * Returns the number of factors for this TaxNumber.  The number is calculated
   *  on creation of the TaxNumber so this just returns the pre-calculated value
   * @return The number of factors for this TaxNumber.
   */
   public final int getFactorCount() { return factors.length; }

   /**
   * Returns the list of factors of this number.
   * @return Array with factors of this number
   */
   public final int[] getFactors() { return factors; }

   /**
   * Returns the integer value of this TaxNumber.
   * @return The integer value of this TaxNumber
   */
   public final int getValue()       { return number; }

   /**
   * Returns a string representation of the TaxNumber
   * @return A string representation of the TaxNumber
   */
   public String toString()
   {
      StringBuffer tmp = new StringBuffer("Number: " + number + "; Factors: ");

      for(int i = 0; i < factors.length; i++)
         tmp.append(factors[i] + " ");

      return tmp.toString();
   }

   /**
   * Testing fuinction
   */
   public static void main(String args[])
   {
      for(int i = 0; i < 100; i++)
         System.out.println(new TaxNumber(i));
   }
}
