package taxman;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;

/**	
* This applet is the main layout manager for the Taxman
*  game.  All the control an setup is done in smaller
*  classes but this one loads files from the server
*  and lays out the big picture.
*
* @Author:	Jacob Crossman
* @Version:	1
*/
public class Taxman extends Applet implements MediaObserver {

   //Color variables for setup
   private static final Color DARK_GREEN   = new Color(0,90,0);
   private static final Color MID_GREEN    = new Color(0,150,0);
   private static final Color LIGHT_GREEN  = new Color(0,210,0);
   private static final Color NEON_GREEN   = new Color(0,255,0);
   private static final Color DARK_GOLD    = new Color(130,100,0);
   private static final Color GOLD         = new Color(240,230,40);
   private static final Color BORDER_COLOR = GOLD;

   //Names for layouts
   private static final String     MAIN_LAYOUT = "MainLayout";
   private static final String     LOAD_LAYOUT = "LoadLayout";
   private              CardLayout layCard     = new CardLayout();

   private Game           g     = null;

   //Upper Panels
   private TitleArea      ta    = null;
   private ScoreArea      score = null;

   //The central tables (where the numbers reside)
   private GatherTable    human = null;
   private ChoiceTable    ct    = null;
   private GatherTable    cpu   = null;

   //Lower Panels
   private ButtonPanel    btns  = null;
   private AdvantagePanel ap    = null;
   private CPUUpdatePanel upd   = null;

   //This holder panel holds the game panel as well as the options
   // panel so they can be swapped on demand.
   private SwapPanel      sp    = null;
   private OptionsPanel   op    = null;
   private EndCanvas      ec    = null;

   //General, outer layout panels
   private Panel          top_section = null;
   private Panel          mid_section = null;
   private Panel          low_section = null;

   //Sound Control object
   private GameSound gs           = new GameSound();
   private boolean   soundsLoaded = false;

   //The two main panels
   // The first panel shows up during media loading and shows progress
   // The second panel shows up after all media have been loaded and
   //  shows the main applet
   private Panel          main_panel  = null;
   private LoadPanel      load_panel  = null;
   
   //ID's for the images in the image array
   private static final int IMG_CHOICE = 0;
   private static final int IMG_TAX    = 1;
   private static final int IMG_WIN    = 2;
   private static final int IMG_LOOSE  = 3;
   private static final int IMG_TIE    = 4;

   //Loader for the media
   private MediaObserver[] media_obs  = { this };        //Array of one element

   //Array of images to load
   private MediaRecord[]   media_list = {
      new MediaRecord("taxman/bag_choice.gif", MediaRecord.GRAPHIC),
      new MediaRecord("taxman/bag_flying.gif", MediaRecord.GRAPHIC),
      new MediaRecord("taxman/winner.gif", MediaRecord.GRAPHIC),
      new MediaRecord("taxman/scrooge.gif", MediaRecord.GRAPHIC),
      new MediaRecord("taxman/tie.gif", MediaRecord.GRAPHIC)
   };


   /**
   * The constructor creates a new game but does not
   *  UI work.
   */
   public Taxman()
   {
      g = new Game();
   }

   /**
   * Initializes and sets up all UI components.
   *  It also loads the media (graphics and audio) files
   */
   public void init()
   {
      //Create the loading panel
      createInitialLayout();

      //Load media files before doing layout
      loadImages();
   }

   /**
   * Creates the layout that is seen while the media is loading
   */
   private void createInitialLayout()
   {
      //Sets the layout for this applet
      setLayout(layCard);

      //Only place one panel in layout so far.
      // place the main panel later
      load_panel = new LoadPanel(media_list.length + gs.getMediaList().length);
      load_panel.setBackground(DARK_GREEN);
      load_panel.setForeground(GOLD);
      add(LOAD_LAYOUT, load_panel);

      //Show the given layout
      layCard.show(this, LOAD_LAYOUT);
   }

   /**
   * Updates the screen when each image is loaded.
   * @param mr The media record that was last loaded
   */
   public void itemLoaded(MediaRecord mr)
   {
      load_panel.newFile(mr.fileName);
      load_panel.repaint();
   }

   /**
   * Returns the image found at the given index of the media array.  The indeces
   *  are named in class constants.
   * @param index the index of the image to retrieve
   * @return the image at the given index of the media list
   */
   private Image findImg(int index) { return (Image)media_list[index].file; }


   public void loadingFinished(MediaRecord[] mr)
   {
      //Once sounds are loaded, we can finally display
      // the applet
      if(soundsLoaded)
      {
         //Innter Panels do the real work
         initializeInnerPanels();
   
         //Outer Panels create structure for inner panels
         initializeOuterPanels();
   
         //Control object sends out events when
         // different actions occur
         initializeControl();
   
         //Finally puts all the components together to form
         // the applet
         createLayout();
      }
      else
      {
         //Get the sounds after the images
         loadSounds();

         soundsLoaded = true;
      }
   }

   /**
   * Creates the inner panels.  These Panels provide
   *  specific, small  functions for the applet as a whole
   */
   private void initializeInnerPanels()
   {
      //The title area
      ta    = new TitleArea("Taxman", GOLD, DARK_GREEN);

      //The area above that tracks the cpu and human scores
      score = new ScoreArea();
      score.setBackground(DARK_GREEN);
      score.setForeground(NEON_GREEN);

      //The central, number choosing area
      ct    = new ChoiceTable(Color.black, findImg(IMG_CHOICE), g);
      ct.setBackground(GOLD);

      //The side that collects the human choices and taxes
      human = new GatherTable(Color.black, findImg(IMG_CHOICE), findImg(IMG_TAX), false);
      human.setBackground(LIGHT_GREEN);

      //The side that collects the cpu choices and taxes
      cpu   = new GatherTable(Color.black, findImg(IMG_CHOICE), findImg(IMG_TAX), true);
      cpu.setBackground(MID_GREEN);

      //The Lower left panel for new game and options buttons
      btns  = new ButtonPanel(g);
      btns.setBackground(LIGHT_GREEN);
      btns.setForeground(DARK_GREEN);

      //The panel on the lower right that monitors the CPU's stats
      upd   = new CPUUpdatePanel();
      upd.setBackground(MID_GREEN);
      upd.setForeground(NEON_GREEN);

      //The panel in the lower middle that shows player advantages
      ap    = new AdvantagePanel();
      ap.setBackground(DARK_GREEN);
      ap.setForeground(NEON_GREEN);

      //the hidden options panel for setting game options
      op    = new OptionsPanel(g);
      op.setBackground(DARK_GREEN);
      op.setForeground(NEON_GREEN);

      //Create a canvas to show the end of the game message
      // and image
      ec    = new EndCanvas(findImg(IMG_WIN), findImg(IMG_LOOSE), findImg(IMG_TIE));
      ec.setForeground(NEON_GREEN);
      ec.setBackground(DARK_GREEN);
   }

   /**
   * This function creates the outer, nesting panels.  These
   *   panels create the general structure of the applet.  Inside
   *   these panels lie the inner panels.
   */
   private void initializeOuterPanels()
   {
      //Top Panels include the Title and score area
      top_section = new Panel();
      top_section.setLayout(new GridLayout(2,0));
      top_section.add(ta);
      top_section.add(score);

      //Mid section includes the number choosing and gathering areas
      //It can be swapped with the options panel
      mid_section = new Panel();
      mid_section.setLayout(new GridLayout(1,0));
      mid_section.add(new BorderPanel(human, 5, 0, 5, 2, BORDER_COLOR));
      mid_section.add(new BorderPanel(ct, 5, 2, 5, 2, BORDER_COLOR));
      mid_section.add(new BorderPanel(cpu, 5, 2, 5, 0, BORDER_COLOR));

      //The swap panel lets the middle section be useful for many things
      sp       = new SwapPanel(mid_section,
                               new BorderPanel(op, 5, 0, 5, 0, GOLD),
                               new BorderPanel(ec, 5, 0, 5, 0, GOLD));

      //The low section holds the button panel (new game and options)
      // the advantage panel and the cpu statistics panel
      low_section = new Panel();
      low_section.setLayout(new GridLayout(0,3));
      low_section.add(btns);
      low_section.add(ap);
      low_section.add(upd);
   }

   /**
   * This function lays out the outer panels giving the most space
   *  to the middle panel because that is where the action occurs
   */
   private void createLayout()
   {
      Panel app = new Panel();

      //Time to layout whole applet.  Use gridbag to
      // give maximum control
      GridBagLayout      gbl = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();

      app.setLayout(gbl);

      //Lay out the top panel (make it small)
      gbc.gridx = 1;
      gbc.gridy = 1;
      gbc.fill  = GridBagConstraints.HORIZONTAL;
      gbl.setConstraints(top_section, gbc);
      app.add(top_section);

      //Lay out the mid panel (give it the most
      // space -- weightx and y)
      gbc.gridx = 1;
      gbc.gridy = 2;
      gbc.fill  = GridBagConstraints.BOTH;
      gbc.weightx = 1.0;
      gbc.weighty = 1.0;
      gbl.setConstraints(sp, gbc);
      app.add(sp);

      //Lay out the lower panel and make it
      // shrink to fit
      gbc.gridx = 1;
      gbc.gridy = 3;
      gbc.fill  = GridBagConstraints.HORIZONTAL;
      gbc.weightx = 0.0;
      gbc.weighty = 0.0;
      gbl.setConstraints(low_section, gbc);
      app.add(low_section);

      //Put border around whole thing and get the show on the road!!
      add(MAIN_LAYOUT, new BorderPanel(app, 5, 5, 5, 5, GOLD));
      layCard.show(this, MAIN_LAYOUT);
   }

   /**
   * Puts all panels that want to be updated about game
   *  events on the waiting list of the game object.
   */
   private void initializeControl()
   {
      g.addPanel(human);         //Human gather table
      g.addPanel(ct);            //The central choice table
      g.addPanel(cpu);           //CPU gather table
      g.addPanel(score);         //The  thin score bar on top
      g.addPanel(upd);           //The update panel on lower right
      g.addPanel(btns);          //The button panel on lower left
      g.addPanel(ap);            //The 'advantage' panel with progress bars
      g.addPanel(sp);            //The swap panel that changes central panel
      g.addPanel(op);            //The options panel
      g.addPanel(ec);            //The end of game canvas that fills center area
      g.addPanel(gs);            //The game sound control
   }

   /**
   * Loads the graphics for the game
   */
   private void loadImages()
   {
      //Load the media files in a separate thread
      Thread thrMedia = new Thread( new MediaLoader( media_list, media_obs, this ) );
      thrMedia.start();
      thrMedia.setPriority(thrMedia.getPriority() - 1);
   }

   /**
   * Loads the sound for the game
   */
   private void loadSounds()
   {
      //Load the media files in a separate thread
      Thread thrMedia = new Thread( new MediaLoader( gs.getMediaList(), media_obs, this ) );
      thrMedia.start();
      thrMedia.setPriority(thrMedia.getPriority() - 1);
   }

   /**
   * Testing function. It creates a frame and runs the applet
   *  in that frame.
   */
   public static void main(String[] args)
   {
      Taxman      applet    = new Taxman();
      TaxmanFrame apFrame   = new TaxmanFrame("Taxman Test");

      apFrame.add(applet, BorderLayout.CENTER);
      apFrame.setSize(800, 600);
      applet.init();
      apFrame.setVisible(true);
      applet.start();

   }
}

/** Special Frame class for testing the applet */
class TaxmanFrame extends Frame
{
   public TaxmanFrame(String title) { super(title); }

   public boolean handleEvent(Event evt)
   {
      if(evt.id == Event.WINDOW_DESTROY)
         System.exit(0);
      else
         return super.handleEvent(evt);

      return true;
   }
}



